@@ -10,14 +10,14 @@ This is intended for both those new to %MythTV and experienced
 with it to get familiar with different aspects of the code base.
 
 If you are just looking for the code formatting standards, 
-see the unofficial mythtv.info page 
-<a href="http://www.mythtv.info/moin.cgi/CodingStandards">
+see the official %MythTV wiki article 
+<a href="http://www.mythtv.org/wiki/index.php/Coding_Standards">
 coding standards</a>. If you are looking for the 
 <a href="http://svn.mythtv.org/trac/wiki/TicketHowTo">bug tracker</a>,
 it can be found on the official pages. 
 If you haven't already, you should subscribe to
 the <a href="http://www.mythtv.org/mailman/listinfo/mythtv-dev/">
-developer mailing list</a>
+developer mailing list</a> and the <a href="http://www.mythtv.org/mailman/listinfo/mythtv-commits/">SVN commits mailing list</a>
 
 If you just stumbled onto the developer pages
 by accident, maybe you want to go to the official
@@ -44,21 +44,23 @@ to documentation on the code itself.
 
 \section libs Libraries
 
-%MythTV is divided up into eight libraries:
+%MythTV is divided up into eleven libraries:
 <dl>
-  <dt>libmyth                <dd>%MythTV Plugin library.
+  <dt>libmyth                <dd>Core %MythTV library. Used by the Plugins.
       The \ref database_subsystem "database",
       \ref audio_subsystem "audio",
       \ref lcd_subsystem "LCD",
       \ref osd_subsystem "OSD",
-      \ref lirc_subsystem "lirc", and the
+      \ref lirc_subsystem "LIRC", and the
       \ref myth_network_protocol "myth network protocol" are supported by libmyth.
-  <dt>libmythtv              <dd>Core %MythTV library.
+  <dt>libmythtv              <dd>%MythTV %TV functionality library.
       The 
-      \ref recorder_subsystem "recorders" and 
+      \ref recorder_subsystem "recorders", \ref video_subsystem "video" and 
       \ref av_player_subsystem "A/V players" are supported by libmythtv.
-  <dt>libavcodec/libavformat <dd>This is the ffmpeg A/V decoding library (aka avlib).
-      <a href="http://ffmpeg.sourceforge.net/documentation.php">Documented Externally</a>.
+  <dt>libmythui              <dd>Main user interface rendering library
+  <dt>libavcodec/libavformat/libavutil
+      <dd>This is the ffmpeg A/V decoding library (aka avlib).
+      <a href="http://ffmpeg.mplayerhq.hu/documentation.html">Documented Externally</a>.
   <dt>libmythmpeg2           <dd>Alternate MPEG-1/2 A/V decoding library.
       <a href="http://libmpeg2.sourceforge.net/">External Website</a>.
   <dt>libmythsamplerate      <dd>Audio resampling library
@@ -66,9 +68,13 @@ to documentation on the code itself.
       We use this to support a different output sample rates than the sample
       rate used in the audio streams we play.
   <dt>libmythsoundtouch      <dd>Pitch preserving audio resampling library.
-      <a href="http://sky.prohosting.com/oparviai/soundtouch/">External Website</a>.
+      <a href="http://www.surina.net/soundtouch/">External Website</a>.
       We use this for the time-stretch feature.
-  <dt>libmythui              <dd>Next Gen UI rendering library, not yet being used.
+  <dt>libmythdvdnav
+      <dd>Used for navigating DVD menus when using the internal player
+  <dt>libmythfreemheg        <dd>UK interactive %TV viewer
+  <dt>libmythlivemedia       <dd>Support for the FreeBox recorder device
+  <dt>libmythupnp            <dd>Initial uPnP (universal Plug and Play) support
 </dl>
 Two libraries libmythmpeg2 and libmythsamplerate appear redundant, but
 libmpeg2 decodes MPEG-2 more quickly than ffmpeg on some systems, and
@@ -79,28 +85,32 @@ to match the hardware sample rate to the A/V streams audio sample rate.
 The database schema is documented here \ref db_schema.
 
 \section apps Applications
-%MythTV contains 12 applications:
+%MythTV contains 14 applications:
 
 <dl>
   <dt>mythbackend      <dd>This is the backend which runs the recorders.
-  <dt>mythfrontend     <dd>This is the frontend which is the main application for viewing
-                           programs and using the %MythTV plugins.
-  <dt>mythtv           <dd>This is the "External Player" used to play videos from
-                           within mythfrontend that are not proper programs,
-                           such as your home movies.
-  <dt>mythtvosd        <dd>This is used externally by programs that want to pop-up
-                            an <i>on screen display</i> in %MythTV while one is watching a
-                           recording.
-  <dt>mythfilldatabase <dd>This is used both internally and externally to
-                           fetch program listings.
-                           <a href="http://tms.tribune.com/">Tribune Media</a> provides
-                           listings in exchange for demographic information in
-                           the USA, and <a href="http://d1.com.au/">D1</a> has
-                           donated free listings in Australia as thanks for the
-                           %MythTV code used in their <i>Home Media Centre</i>
-                           product. Other markets are served by the
-                           <a href="http://membled.com/work/apps/xmltv/xmltv">XMLTV</a>
-                           web spiders.
+  <dt>mythfrontend
+      <dd>This is the frontend which is the main application
+          for viewing programs and using the %MythTV plugins.
+  <dt>mythtv-setup     <dd>This is the program which sets up the database
+                           to use a machine as a backend server.
+  <dt>mythtv           <dd>This was an "External Player" used to play videos
+                           from within mythfrontend. Setting the player command
+                           to "internal" does the same thing now. This is handy
+                           for testing the audio and videoout code, though.
+  <dt>mythtvosd
+      <dd>This is used externally by programs that want to pop-up an
+          <i>on screen display</i> in %MythTV while one is watching a recording.
+  <dt>mythfilldatabase
+      <dd>This is used both internally and externally to fetch program listings.
+          <a href="http://tms.tribune.com/">Tribune Media</a> provides
+          listings in exchange for demographic information in the USA,
+          and Australia uses a
+          <a href="http://minnie.tuhs.org/twiki/bin/view/TVGuide">
+          community-driven TV guide</a> originally developed for OzTiVo.
+          Other markets are served by the
+          <a href="http://membled.com/work/apps/xmltv/xmltv">XMLTV</a>
+          web spiders.
   <dt>mythtranscode    <dd>This is used both internally and externally to
                            transcode videos from one format to another. 
                            This is used to shrink HDTV programs to lower
@@ -117,20 +127,29 @@ The database schema is documented here \ref db_schema.
                            programs to record based on the first letter of
                            the program name.
   <dt>mythuitest       <dd>This is a test program for libmythui development.
-  <dt>mythlcd          <dd>This is a test program for %MythTV %LCD support.
+  <dt>mythlcdserver    <dd>This is an interface between a number of Myth
+                           clients and a small text display (LCDProc server).
+  <dt>mythwelcome/mythshutdown
+      <dd>These programs manage Power Saving (shutdown/wakeup) on your Myth PCs.
 </dl>
 
 \section fe_plugins Frontend Plugins
 <dl>
+  <dt>mytharchive <dd>Creates themed Video DVDs from recordings
+                      (and other video files).
   <dt>mythbrowser <dd>Provides a simple web browser.
-  <dt>mythdvd     <dd>Launches DVD players such as Xine and MPlayer.
-  <dt>mythgame    <dd>Launches the xmame classic game system emulator.
-  <dt>mythnews    <dd>Browses RSS news feeds.
-  <dt>mythweather <dd>Presents your local weather report.
+  <dt>mythcontrols<dd>Editor for Myth Key bindings, \e et \e c.
+  <dt>mythflix    <dd>
   <dt>mythgallery <dd>A simple picture viewer for your %TV.
+  <dt>mythgame    <dd>Launches the xmame classic game system emulator.
+  <dt>mythmovies  <dd>
   <dt>mythmusic   <dd>A simple music player for your %TV.
+  <dt>mythnews    <dd>Browses RSS news feeds.
   <dt>mythphone   <dd>SIP based video phone.
-  <dt>mythvideo   <dd>Video Browser for non-%MythTV recordings.
+  <dt>mythvideo   <dd>Launch DVD players, and a Video Browser for other files
+                      (non-%MythTV recordings).
+  <dt>mythweather <dd>Presents your local weather report.
+  <dt>mythzoneminder<dd>
 </dl>
 
 \section be_plugins Backend Plugins
@@ -138,6 +157,14 @@ The database schema is documented here \ref db_schema.
   <dt>mythweb     <dd>Provides a PHP based web pages to control mythbackend.
 </dl>
 
+\section spt_scripts Support Scripts
+These tools are in the contrib directory of the source tree:
+<dl>
+  <dt>osx-packager.pl   <dd>Downloads and builds all dependencies, then the
+                            source, of %MythTV and all the official plugins,
+                            on Mac OS 10.3 thru 10.5
+  <dt>win32-packager.pl <dd>Similar tool for Windows XP and Vista
+</dl>
  */
 
 /** \defgroup database_subsystem    Database Subsystem
@@ -168,13 +195,15 @@ from the database automagically when used in %MythTV's window classes.
     \todo No one is working on documenting the LIRC Subsystem
  */
 
-/** \defgroup myth_network_protocol Myth Network Protocol
-    \todo No one is working on documenting the myth network protocol
+/** \defgroup video_subsystem       Video Subsystem
+    \todo No one is working on documenting the video subsystem
  */
 
 /** \defgroup recorder_subsystem    Recorder Subsystem
+This line is filler that is ignored by Doxygen.
+
 TVRec is the main class for handling recording. 
-\todo +++It is passed a ProgramInfo for the current and next recordings IS THIS TRUE? +++,
+It is passed a ProgramInfo for the current and next recordings,
 and in turn creates three main worker classes:
 <dl>
   <dt>RecorderBase <dd>Recordings from device into RingBuffer.
@@ -183,7 +212,7 @@ and in turn creates three main worker classes:
 </dl>
 
 %TVRec also presents the public face of a recordings to the 
-Myth Network Protocol, and hence the rest of %MythTV. This
+\ref myth_network_protocol, and hence the rest of %MythTV. This
 means that any call to the %RecorderBase, %RingBuffer, or
 %ChannelBase is marshalled via methods in the %TVRec.
 
@@ -224,12 +253,16 @@ is ready for commands. Finally, the "ChangingState" state
 tells us that a state change request is pending, so other
 state changing commands should not be issued.
 
+\todo Check if the TVRec is actually still passed a ProgramInfo for the current and next recordings
+
+\todo Document the massive changes Daniel has made in the Vid and MultiRec branches
  */
 
 /** \defgroup av_player_subsystem   A/V Player Subsystem
+This line is filler that is ignored by Doxygen.
 
 TV is the main class for handling playback.
-It instanciates several important classes:
+It instantiates several important classes:
 <dl>
 <dt>NuppelVideoPlayer <dd>Decodes audio and video from the RingBuffer, then plays and displays them, resp.
 <dt>RemoteEncoder     <dd>Communicates with TVRec on the backend.
@@ -266,7 +299,266 @@ state changing commands should not be issued.
 
  */
 
+/** \defgroup plugin_arch   Plugin Architecture
+This line is filler that is ignored by Doxygen.
+
+MythPlugins are shared object files (<I>i.e.</I> libraries) which are loaded
+from a specific directory (<I>%e.g.</I> /usr/local/lib/mythtv/plugins).
+Currently, all plugins are written in the C++ language, but there is nothing
+preventing other languages being used (the functions are in the C name space).
+
+int mythplugin_init(const char *libversion); is invoked whenever mythfrontend
+is started. This typically handles upgrading any database records - it will be
+the first method called after a new version of the plugin has been installed.
+
+int mythplugin_run(); is invoked when the action attribute of a menu item
+requests it via the PLUGIN directive. This is when the user chooses to enter
+the plugin from the main menu or the appropriate submenu.
+
+int mythplugin_config(); is invoked when the action attribute of a menu item
+requests it via the CONFIGPLUGIN directive. This should be when the users
+wishes to change the plugin's settings (in the main Setup menu).
+
+Other plugin functions are listed in the file \link mythpluginapi.h \endlink
+(see also the \link MythPlugin \endlink
+ and \link MythPluginManager \endlink classes.)
+
+*/
+
+/** \defgroup mtd                   MTD (the Myth Transcoding Daemon)
+This line is filler that is ignored by Doxygen.
+
+The %MTD is a simple program that is used by the MythVideo plugin.
+
+It currently \b only transcodes tracks from DVDs, but could in the
+future also rip CD/VCD tracks, transcode %TV recordings, \e et \e cetera.
+
+The DVD Rip screen will offer to start %MTD if it is not running.
+It does \b not currently shut it down afterwards, though.
+
+By default, the %MTD listens on port 2442, for commands of this format:
+
+\verbatim
+halt
+quit
+shutdown
+\endverbatim
+- Tells the %MTD to wait for all jobs to finish, stop listening, and exit.
+
+\verbatim
+hello
+\endverbatim
+- Send a trivial response if the %MTD is listening.
+
+\verbatim
+status
+\endverbatim
+<B>\verbatim
+status dvd summary 1
+status dvd job 0 overall 0.0262568 ISO copy of UNTITLED
+status dvd job 0 subjob 0.0262568 Ripping to file ~ 00:08/05:04
+status dvd complete
+\endverbatim</B>
+- Sends a response listing the status of all jobs.
+
+\verbatim
+media
+\endverbatim
+<B>\verbatim
+media dvd summary 1 UNTITLED
+media dvd title 1 1 1 0 5 27 8
+media dvd title-audio 1 1 2 nl ac3 2Ch
+media dvd complete
+\endverbatim</B>
+- Sends a detailed list of titles/tracks on the current disk.
+
+\verbatim
+job dvd TITLE AUDIOTRK QUALITY AC3 SUBTITLETRK DESTPATH
+\endverbatim
+\verbatim
+e.g. job dvd 1 1 -1 0 -1 /myth/video/UNTITLED
+\endverbatim
+- Starts a transcoding job. TITLE, AUDIOTRK and SUBTITLETRK are integers
+  to select the track details. AC3 is 0 or 1. DESTPATH is the output file.
+  QUALITY is one of enum \ref RIP_QUALITIES (-1, 0 or 1).
+
+\verbatim
+abort dvd job NNNN
+\endverbatim
+- Stops the job identified by the job number NNNN.
+
+\verbatim
+use dvd PATH
+\endverbatim
+- Check dvd PATH is usable, and if so set the current drive to it.
+
+\verbatim
+no dvd
+\endverbatim
+- Forget about the current dvd path, ending any jobs that are trying to read it.
+
+\verbatim
+no dvd PATH
+\endverbatim
+- End any jobs that are reading from dvd PATH, and if it is the current drive,
+  forget about it.
+ */
+
+/** \defgroup myth_media            Myth Media Manager
+This line is filler that is ignored by Doxygen.
+
+The Myth Media Manager is a thread in the frontend which looks for any
+changes to removable media, and sends events to any Frontend Plugins
+which are interested in that media.
+
+At startup, it creates a list of MythMediaDevice objects (currently
+MythHDD or MythCDROM and its subclasses) for each removable device
+configured on the system. A runtime loop then monitors each of these
+via its checkMedia() method.
+
+When any of these devices change status, a MediaEvent object is created.
+If the device has a status which is usable, the window jumps to the main menu
+(to allow the registered jump to work correctly), and the event is dispatched
+to the relevant plugin's registered media handler. If the device status is
+unusable (<I>%e.g.</I> ejected), the plugin's media handler is called directly
+(so it can "forget" about this device).
+
+The following tables show
+typical status transitions for CD/DVD and USB flash drive devices:
+
+\verbatim
+      CD/DVD                     USB/FireWire HDD
+State        Action           State         Action
+--------------------------------------------------
+NODISK                        NODISK
+             eject                          attach
+OPEN                          USEABLE
+       insert disk, close                   detach
+USEABLE                       UNPLUGGED
+             mount
+MOUNTED
+            unmount
+NOTMOUNTED
+--------------------------------------------------
+\endverbatim
+ */
+
+/** \defgroup myth_network_protocol Myth Network Protocol
+This line is filler that is ignored by Doxygen.
+
+The MythTV backend process currently opens sockets for three different types
+of commands; a custom protocol (by default at port 6543),
+an HTML server (by default <A HREF="http://127.0.0.1:6544">
+http://127.0.0.1:6544
+</A> for the status, and other services under that, like
+< HREF="http://127.0.0.1:6544/Myth/GetMusic?Id=1">
+http://127.0.0.1:6544/Myth/GetMusic?Id=1
+</A>), and a UPnP media server (several ports that I cannot
+remember right now that throw around lots of little bits of XML :-)
+
+The custom protocol is an ASCII encoded length and command string.
+Command sequences can be easily sent to the backend using telnet.
+<I>%e.g.</I> \verbatim telnet 127.0.0.1 6543
+Trying 127.0.0.1...
+Connected to localhost.
+Escape character is '^]'.\endverbatim
+<B>\verbatim21      MYTH_PROTO_VERSION 36 23     ANN Playback hostname 1   10   QUERY_LOAD       4DONE\endverbatim</B>
+\verbatim13      ACCEPT[]:[]362       OK34      0.919922[]:[]0.908203[]:[]0.856445Connection closed by foreign host.\endverbatim
+The command string is prefixed by 8 characters, containing the length
+of the forthcoming command. This can be justified in any way
+(as the above example shows)
+
+The backend responds with a length, and the response to the command.
+This can be numbers (up to 32 bit, represented in ASCII),
+a single string, or an ASCII encoding of a QStringList.
+The 5 byte sequence "[]:[]" seperates items in the stringlist.
+Any 64 bit numbers are represented as a stringlist of two 32bit words
+(MSB first).
+
+\section commands Commands
+
+There are three main types of networking interactions in MythTV;
+identification commands (which tell the backend about this client),
+query commands that are sent to the master backend
+(<I>%e.g.</I> listing recordings or viewing guide data), and
+file streaming commands (when a frontend is watching or editing a recording).
+
+Until a client is identified to the backend (via the ANN commands),
+any of the query or file streaming commands will silently fail.
+
+The following summarises some of these commands.
+For a full understanding of all the commands, either read the source code
+(programs/mythbackend/mainserver.cpp), or look on the Wiki
+(http://www.mythtv.org/wiki/index.php/Myth_Protocol_Command_List).
+
+ */
+
+/** \defgroup myth_startup Myth startup sequence
+This line is filler that is ignored by Doxygen.
+
+Most MythTV programs follow a common sequence:
+<ol>
+  <li>Process (parse) command-line arguments</li>
+  <li>Create a MythContext object, which stores paths for later location
+      of runtime assets (filters/fonts/plugins/themes/translations)</li>
+  <li>(optionally) Create a UPnP client or server</li>
+  <li>Initialise the MythContext, which:</li>
+  <ul>
+    <li>Tries to find a database on localhost,
+        or on the host specified in mysql.txt,</li>
+    <li>Tries to locate exactly one backend host via UPnP,
+        to find its database,</li>
+    <li>If possible, displays a list of all backends located via UPnP
+        for the user to choose from, or</li>
+    <li>Fails</li>
+  </ul>
+  <li>Create the main window/screen, display themed menus, <i>et c.</i></li>
+</ol>
+(examine program/*/main.cpp, and libs/libmyth/mythcontext.cpp,
+for further detail).
+
+<p>
+
+The "runtime assets" mentioned above are stored in a number of well-known
+locations. The following methods in MythContext allow programs and plugins
+to access these assets:
+<ol>
+  <li>GetInstallPrefix() returns the value of MCP's m_installprefix variable,
+      which is either the runtime env. var. $MYTHTVDIR or the compile-time var.
+      RUNPREFIX. If these are relative paths, it is initialised relative to the
+      binary location. The value is used thus:
+  <ul>
+    <li>GetInstallPrefix() + /share/mythtv/ = GetShareDir(), GetFontsDir()</li>
+    <li>GetInstallPrefix() + /share/mythtv/themes/ = GetThemesParentDir()</li>
+    <li>GetInstallPrefix() + /share/mythtv/i18n/   = GetTranslationsDir()</li>
+    <li>GetInstallPrefix() + /share/mythtv/mytharchive</li>
+    <li>GetInstallPrefix() + /share/mythtv/mytharchive/themes</li>
+    <li>GetInstallPrefix() + /share/mythtv/mytharchive/scripts</li>
+    <li>GetInstallPrefix() + /share/mythtv/mythflix/scripts</li>
+    <li>GetInstallPrefix() + /share/mythtv/mythnews</li>
+    <li>GetInstallPrefix() + /share/mythtv/mythvideo/scripts</li>
+    <li>GetInstallPrefix() + /share/mythtv/mythweather</li>
+    <li>GetInstallPrefix() + /share/mythtv/mythweather/scripts</li>
+    <li>GetInstallPrefix() + /bin/ignyte</li>
+    <li>GetInstallPrefix() + /bin/mythfilldatabase</li>
+    <li>GetInstallPrefix() + /bin/mtd</li>
+    <li>GetInstallPrefix() + /lib/mythtv/ = GetLibraryDir()</li>
+    <li>GetInstallPrefix() + /lib/mythtv/plugins/ = GetPluginsDir()</li>
+    <li>GetInstallPrefix() + /lib/mythtv/filters/ = GetFiltersDir()</li>
+  </ul></li>
+
+  <li>GetConfDir() returns the value of the runtime env. var. $MYTHCONFDIR,
+      or $HOME/.mythtv.</li>
+
+  <li>mysql.txt is loaded from GetShareDir(), GetInstallPrefix() + /etc/mythtv,
+      GetConfDir(), and the current directory. Later files override the values
+      from earlier ones.</li>
+</ol>
+
+ */
+
 /** \defgroup qobject_dangers QObject is dangerous for your health
+This line is filler that is ignored by Doxygen.
 
 QObject derived classes can be quite useful, they can send and receive
 signals, get keyboard events, translate strings into another language
@@ -384,6 +676,7 @@ wrong slots.
 */
 
 /** \defgroup profiling_mythtv    Profiling MythTV
+This line is filler that is ignored by Doxygen.
 
 You can use any sampling profiler to profile %MythTV, the most
 popular GPL one is OProfile and this section tells you how to
@@ -536,3 +829,23 @@ This includes information on constructing callgraphs, producing
 an annotated versions of the code, and saving results.
 
 */
+
+// The file ac3.h in ffmpeg contains no Doxygen markup,
+// but for some reason generates the following modules:
+// * Coded elements
+// * Derived values
+//
+// They add nothing useful to this programmer doco, so I camouflage these
+// by manually defining the block here. Same thing for the few modules
+// in ffmpeg that are documented; Macroblock and VC9 bitplanes from vc1.c,
+// Monkey Audio from apedec.c, multithreaded slicing from h264.h,
+
+/**
+ @defgroup coded   .
+ @defgroup derived .
+ @defgroup block   .
+ @defgroup std_mb  .
+ @defgroup rangecoder .
+ @defgroup multithreading .
+ @defgroup bitplane .
+ */
