@@ -7,6 +7,7 @@
 // POSIX C headers
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/time.h>
 #include <unistd.h>
 #include <fcntl.h>
 #include <pthread.h>
@@ -14,7 +15,6 @@
 // Qt headers
 #include <qsocket.h>
 #include <qfile.h>
-#include <qregexp.h>
 #include <qapplication.h>
 #include <qdatetime.h>
 #include <qfileinfo.h>
@@ -30,6 +30,7 @@ using namespace std;
 #include "livetvchain.h"
 #include "DVDRingBuffer.h"
 #include "util.h"
+#include "compat.h"
 
 #ifndef O_STREAMING
 #define O_STREAMING 0
@@ -39,7 +40,13 @@ using namespace std;
 #define O_LARGEFILE 0
 #endif
 
-const uint RingBuffer::kBufferSize = 10 * 256 * 1024;
+#ifndef O_BINARY
+#define O_BINARY 0
+#endif
+
+const uint RingBuffer::kBufferSize = 3 * 1024 * 1024;
+
+#define CHUNK 32768 /* readblocksize increments */
 
 #define PNG_MIN_SIZE   20 /* header plus one empty chunk */
 #define NUV_MIN_SIZE  204 /* header size? */
@@ -91,9 +98,9 @@ RingBuffer::RingBuffer(const QString &lfilename,
       rbrpos(0),                rbwpos(0),
       internalreadpos(0),       ateof(false),
       readsallowed(false),      wantseek(false), setswitchtonext(false),
-      rawbitrate(8000),         playspeed(1.0f),
-      fill_threshold(-1),       fill_min(-1),
-      readblocksize(128000),    wanttoread(0),
+      rawbitrate(4000),         playspeed(1.0f),
+      fill_threshold(65536),    fill_min(-1),
+      readblocksize(CHUNK),     wanttoread(0),
       numfailures(0),           commserror(false),
       dvdPriv(NULL),            oldfile(false),
       livetvchain(NULL),        ignoreliveeof(false),
@@ -115,7 +122,8 @@ RingBuffer::RingBuffer(const QString &lfilename,
         return;
     }
 
-    OpenFile(filename, read_retries);
+    if (read_retries != (uint)-1)
+        OpenFile(filename, read_retries);
 }
 
 /** \fn check_permissions(const QString&)
@@ -162,91 +170,93 @@ void RingBuffer::OpenFile(const QString &lfilename, uint retryCount)
 
     bool is_local = false;
     bool is_dvd = false;
-    if ((filename.left(7) == "myth://") &&
-        (filename.length() > 7 ))
-    {
-        QString local_pathname = gContext->GetSetting("RecordFilePrefix");
-        int hostlen = filename.find(QRegExp("/"), 7);
+    (void) is_dvd; // not used when frontend is disabled.
 
-        if (hostlen != -1)
-        {
-            local_pathname += filename.right(filename.length() - hostlen);
-
-            QFile checkFile(local_pathname);
-            if (checkFile.exists())
-            {
-                is_local = true;
-                filename = local_pathname;
-            }
-        }
-    }
+    if ((filename.left(1) == "/") ||
+        (QFile::exists(filename)))
+        is_local = true;
+#ifdef USING_FRONTEND
     else if (filename.left(4) == "dvd:")
     {
         is_dvd = true;
         dvdPriv = new DVDRingBufferPriv();
         startreadahead = false;
-        int pathLen = filename.find(QRegExp("/"), 4);
-        if (pathLen != -1)
-        {
-            QString tempFilename = filename.right(filename.length() -  pathLen);
 
-            QFile checkFile(tempFilename);
-            if (!checkFile.exists())
-                filename = "/dev/dvd";
-            else
-                filename = tempFilename;
-        }
+        if (filename.left(6) == "dvd://")  // 'Play DVD' sends "dvd:/" + dev
+            filename.remove(0,5);          //             e.g. "dvd://dev/sda"
+        else                               // Less correct URI "dvd:" + path
+            filename.remove(0,4);          //             e.g. "dvd:/videos/ET"
+
+        if (QFile::exists(filename))
+            VERBOSE(VB_PLAYBACK, "OpenFile() trying DVD at " + filename);
         else
         {
             filename = "/dev/dvd";
         }
     }
-    else
-        is_local = true;
+#endif // USING_FRONTEND
 
     if (is_local)
     {
         char buf[kReadTestSize];
-        while (openAttempts > 0)
+        int timetowait = 500 * openAttempts;
+        int lasterror = 0;
+
+        MythTimer openTimer;
+        openTimer.start();
+
+        while (openTimer.elapsed() < timetowait)
         {
-            openAttempts--;
-                
-            fd2 = open(filename.ascii(), O_RDONLY|O_LARGEFILE|O_STREAMING);
+            lasterror = 0;
+            fd2 = open(filename.local8Bit(),
+                       O_RDONLY|O_LARGEFILE|O_STREAMING|O_BINARY);
                 
             if (fd2 < 0)
             {
                 if (!check_permissions(filename))
                     break;
 
-                VERBOSE(VB_IMPORTANT,
-                        QString("Could not open %1.  %2 retries remaining.")
-                        .arg(filename).arg(openAttempts));
-
-                usleep(500000);
+                lasterror = 1;
+                usleep(1000);
             }
             else
             {
                 int ret = read(fd2, buf, kReadTestSize);
                 if (ret != (int)kReadTestSize)
                 {
-                    VERBOSE(VB_IMPORTANT, LOC +
-                            QString("Invalid file (fd %1) when opening '%2'.")
-                            .arg(fd2).arg(filename) + 
-                            QString(" %2 retries remaining.")
-                            .arg(openAttempts));
-
+                    lasterror = 2;
                     close(fd2);
                     fd2 = -1;
-                    usleep(500000);
+                    usleep(1000);
                 }
                 else
                 {
                     lseek(fd2, 0, SEEK_SET);
+#ifdef HAVE_POSIX_FADVISE
+                    posix_fadvise(fd2, 0, 0, POSIX_FADV_SEQUENTIAL);
+#endif
                     openAttempts = 0;
+                    break;
                 }
             }
         }
 
+        switch (lasterror)
+        {
+            case 1:
+                VERBOSE(VB_IMPORTANT, LOC +
+                        QString("Could not open %1.").arg(filename));
+                break;
+            case 2:
+                VERBOSE(VB_IMPORTANT, LOC +
+                        QString("Invalid file (fd %1) when opening '%2'.")
+                        .arg(fd2).arg(filename));
+                break;
+            default:
+                break;
+        }
+
+
         QFileInfo fileInfo(filename);
         if (fileInfo.lastModified().secsTo(QDateTime::currentDateTime()) >
             30 * 60)
@@ -254,11 +264,13 @@ void RingBuffer::OpenFile(const QString &lfilename, uint retryCount)
             oldfile = true;
         }
     }
+#ifdef USING_FRONTEND
     else if (is_dvd)
     {
         dvdPriv->OpenFile(filename);
         readblocksize = DVD_BLOCK_SIZE * 62;
     }
+#endif // USING_FRONTEND
     else
     {
         remotefile = new RemoteFile(filename);
@@ -271,6 +283,14 @@ void RingBuffer::OpenFile(const QString &lfilename, uint retryCount)
             remotefile = NULL;
         }
     }
+
+    setswitchtonext = false;
+    ateof = false;
+    commserror = false;
+    numfailures = 0;
+
+    rawbitrate = 4000;
+    CalcReadAheadThresh();
 }
 
 /** \fn RingBuffer::IsOpen(void) const
@@ -278,11 +298,11 @@ void RingBuffer::OpenFile(const QString &lfilename, uint retryCount)
  */
 bool RingBuffer::IsOpen(void) const
 { 
-#ifdef HAVE_DVDNAV
+#ifdef USING_FRONTEND
     return tfw || (fd2 > -1) || remotefile || (dvdPriv && dvdPriv->IsOpen());
-#else // if !HAVE_DVDNAV
-    return tfw || (fd2 > -1) || remotefile; 
-#endif // !HAVE_DVDNAV
+#else // if !USING_FRONTEND
+    return tfw || (fd2 > -1) || remotefile;
+#endif // !USING_FRONTEND
 }
 
 /** \fn RingBuffer::~RingBuffer(void)
@@ -311,11 +331,15 @@ RingBuffer::~RingBuffer(void)
         fd2 = -1;
     }
     
+#ifdef USING_FRONTEND
     if (dvdPriv)
     {
         delete dvdPriv;
     }
-    
+#endif // USING_FRONTEND    
+
+    pthread_rwlock_unlock(&rwlock);
+    pthread_rwlock_destroy(&rwlock);
 }
 
 /** \fn RingBuffer::Start(void)
@@ -414,7 +438,7 @@ int RingBuffer::safe_read(int fd, void *data, uint sz)
 
             // 3 second timeout with usleep(60000), 
             // or 0.12 seconds if it's an old, unmodified file.
-            if (zerocnt >= ((oldfile) ? 2 : (livetvchain) ? 6 : 50))
+            if (zerocnt >= ((oldfile) ? 2 : (livetvchain) ? 6 : 40))
             {
                 break;
             }
@@ -504,45 +528,51 @@ void RingBuffer::UpdatePlaySpeed(float play_speed)
  */
 void RingBuffer::CalcReadAheadThresh(void)
 {
-    const uint KB32  =  32*1024;
-    const uint KB128 = 128*1024;
-    const uint KB256 = 256*1024;
-    const uint KB512 = 512*1024;
     uint estbitrate;
 
-    wantseek = true;
     pthread_rwlock_wrlock(&rwlock);
-
-    estbitrate     = (uint) max(abs(rawbitrate * playspeed), 0.5f * rawbitrate);
-    estbitrate     = min(rawbitrate * 3, estbitrate);
     wantseek       = false;
     readsallowed   = false;
+    readblocksize  = CHUNK;
+
+    // loop without sleeping if the buffered data is less than this
+    fill_threshold = CHUNK * 2;
     fill_min       = 1;
-    readblocksize  = (estbitrate > 9000)  ? KB256 : KB128;
-    readblocksize  = (estbitrate > 18000) ? KB512 : readblocksize;
 
-    uint  secs_thr = 300; // seconds of buffering desired
-    float secs_min = 0.1; // minumum seconds of buffering before allowing read
+#ifdef USING_FRONTEND
+    if (dvdPriv)
+    {
+        const uint KB32  =  32*1024;
+        const uint KB64  =  64*1024;
+        const uint KB128 = 128*1024;
+        const uint KB256 = 256*1024;
+        const uint KB512 = 512*1024;
 
-    // set basic fill_threshold based on bitrate
-    fill_threshold = (estbitrate * secs_thr) >> 3; // >>3 => bits -> bytes
-    // extra buffering for remote files
-    fill_threshold = fill_threshold + ((remotefile) ? KB256 : 0);
-    // make the fill_threshold at least one block
-    fill_threshold = max(readblocksize, fill_threshold);
+        estbitrate     = (uint) max(abs(rawbitrate * playspeed),
+                                    0.5f * rawbitrate);
+        estbitrate     = min(rawbitrate * 3, estbitrate);
+        readblocksize  = (estbitrate > 2500)  ? KB64  : KB32;
+        readblocksize  = (estbitrate > 5000)  ? KB128 : readblocksize;
+        readblocksize  = (estbitrate > 9000)  ? KB256 : readblocksize;
+        readblocksize  = (estbitrate > 18000) ? KB512 : readblocksize;
 
-    // set the minimum buffering before allowing ffmpeg read
-    fill_min        = (uint) ((estbitrate * secs_min) * 0.125f);
-    // make this a multiple of ffmpeg block size..
-    fill_min        = ((fill_min / KB32) + 1) * KB32;
+        // minumum seconds of buffering before allowing read
+        float secs_min = 0.1;
+
+        // set the minimum buffering before allowing ffmpeg read
+        fill_min        = (uint) ((estbitrate * secs_min) * 0.125f);
+        // make this a multiple of ffmpeg block size..
+        fill_min        = ((fill_min / KB32) + 1) * KB32;
+    }
+#endif // USING_FRONTEND    
+
+    pthread_rwlock_unlock(&rwlock);
 
     VERBOSE(VB_PLAYBACK, LOC +
             QString("CalcReadAheadThresh(%1 KB)\n\t\t\t -> "
                     "threshhold(%2 KB) min read(%3 KB) blk size(%4 KB)")
             .arg(estbitrate).arg(fill_threshold/1024)
             .arg(fill_min/1024).arg(readblocksize/1024));
-
-    pthread_rwlock_unlock(&rwlock);
 }
 
 /** \fn RingBuffer::ReadBufFree(void) const
@@ -578,6 +608,7 @@ int RingBuffer::ReadBufAvail(void) const
 void RingBuffer::ResetReadAhead(long long newinternal)
 {
     readAheadLock.lock();
+    readblocksize = CHUNK;
     rbrpos = 0;
     rbwpos = 0;
     internalreadpos = newinternal;
@@ -596,10 +627,10 @@ void RingBuffer::StartupReadAheadThread(void)
 {
     readaheadrunning = false;
 
+    readAheadRunningCondLock.lock();
     pthread_create(&reader, NULL, StartReader, this);
-
-    while (!readaheadrunning)
-        usleep(50);
+    readAheadRunningCond.wait(&readAheadRunningCondLock);
+    readAheadRunningCondLock.unlock();
 }
 
 /** \fn RingBuffer::KillReadAheadThread(void)
@@ -689,14 +720,23 @@ void RingBuffer::ReadAheadThread(void)
     int used = 0;
     int loops = 0;
 
+    struct timeval lastread, now;
+    gettimeofday(&lastread, NULL);
+    const int KB640 = 640*1024;
+    int readtimeavg = 300;
+    int readinterval;
+
     pausereadthread = false;
 
-    readAheadBuffer = new char[kBufferSize + 256000];
+    readAheadBuffer = new char[kBufferSize + KB640];
 
     ResetReadAhead(0);
     totfree = ReadBufFree();
 
     readaheadrunning = true;
+    readAheadRunningCondLock.lock();
+    readAheadRunningCond.wakeAll();
+    readAheadRunningCondLock.unlock();
     while (readaheadrunning)
     {
         if (pausereadthread || writemode)
@@ -727,25 +767,58 @@ void RingBuffer::ReadAheadThread(void)
         loops = 0;
 
         pthread_rwlock_rdlock(&rwlock);
-        if (totfree > readblocksize && !commserror)
+        if (totfree > readblocksize && !commserror && !ateof && !setswitchtonext)
         {
             // limit the read size
             totfree = readblocksize;
 
+            // adapt blocksize
+            gettimeofday(&now, NULL);
+            readinterval = (now.tv_sec  - lastread.tv_sec ) * 1000 +
+                           (now.tv_usec - lastread.tv_usec) / 1000;
+
+            readtimeavg = (readtimeavg * 9 + readinterval) / 10;
+
+            if (readtimeavg < 200 && readblocksize < KB640)
+            {
+                readblocksize += CHUNK;
+                //VERBOSE(VB_PLAYBACK,
+                //    QString("Avg read interval was %1 msec. %2K block size")
+                //            .arg(readtimeavg).arg(readblocksize/1024));
+                readtimeavg = 300;
+            }
+            else if (readtimeavg > 400 && readblocksize > CHUNK)
+            {
+                readblocksize -= CHUNK;
+                //VERBOSE(VB_PLAYBACK,
+                //    QString("Avg read interval was %1 msec. %2K block size")
+                //            .arg(readtimeavg).arg(readblocksize/1024));
+                readtimeavg = 300;
+            }
+            lastread = now;
+
             if (rbwpos + totfree > kBufferSize)
                 totfree = kBufferSize - rbwpos;
 
+            if (internalreadpos == 0)
+                totfree = fill_min;
+
             if (remotefile)
             {
+                if (livetvchain && livetvchain->HasNext())
+                    remotefile->SetTimeout(true);
+
                 ret = safe_read(remotefile, readAheadBuffer + rbwpos,
                                 totfree);
                 internalreadpos += ret;
             }
+#ifdef USING_FRONTEND
             else if (dvdPriv)
             {                        
                 ret = dvdPriv->safe_read(readAheadBuffer + rbwpos, totfree);
                 internalreadpos += ret;
             }
+#endif // USING_FRONTEND
             else
             {
                 ret = safe_read(fd2, readAheadBuffer + rbwpos, totfree);
@@ -785,14 +858,22 @@ void RingBuffer::ReadAheadThread(void)
             totfree = 0;
         }
 
-        if (!readsallowed && used >= fill_min)
+        if (!readsallowed && (used >= fill_min || setswitchtonext))
+        {
             readsallowed = true;
+            //VERBOSE(VB_PLAYBACK, QString("reads allowed (%1 %2)").arg(used)
+            //                                                    .arg(fill_min));
+        }
+        //else if (!readsallowed)
+        //    VERBOSE(VB_PLAYBACK, QString("buffering (%1 %2 %3)").arg(used)
+        //                                                        .arg(fill_min)
+        //                                                        .arg(ret));
 
         if (readsallowed && used < fill_min && !ateof && !setswitchtonext)
         {
             readsallowed = false;
-            VERBOSE(VB_GENERAL, QString ("rebuffering (%1 %2)").arg(used)
-                                                               .arg(fill_min));
+            //VERBOSE(VB_GENERAL, QString ("rebuffering (%1 %2)").arg(used)
+            //                                                   .arg(fill_min));
         }
 
         if (readsallowed || stopreads)
@@ -808,8 +889,11 @@ void RingBuffer::ReadAheadThread(void)
 
         pthread_rwlock_unlock(&rwlock);
 
-        if ((used >= fill_threshold || wantseek) && !pausereadthread)
+        if ((used >= fill_threshold || wantseek || ateof || setswitchtonext) &&
+            !pausereadthread)
+        {
             usleep(500);
+        }
     }
 
     delete [] readAheadBuffer;
@@ -824,14 +908,28 @@ long long RingBuffer::SetAdjustFilesize(void)
     return readAdjust;
 }
 
-/** \fn RingBuffer::ReadFromBuf(void*, int)
+int RingBuffer::Peek(void *buf, int count)
+{
+    // really only works with readahead, but DVD doesn't readahead
+    if (!readaheadrunning)
+    {
+        int ret = Read(buf, count);
+        Seek(0, SEEK_SET);
+        return ret;
+    }
+
+    return ReadFromBuf(buf, count, true);
+}
+
+/**
  *  \brief Reads from the read-ahead buffer, this is called by
  *         Read(void*, int) when the read-ahead thread is running.
  *  \param buf   Pointer to where data will be written
  *  \param count Number of bytes to read
+ *  \param peek  If true, don't increment read count
  *  \return Returns number of bytes read
  */
-int RingBuffer::ReadFromBuf(void *buf, int count)
+int RingBuffer::ReadFromBuf(void *buf, int count, bool peek)
 {
     if (commserror)
         return 0;
@@ -853,15 +951,15 @@ int RingBuffer::ReadFromBuf(void *buf, int count)
                  VERBOSE(VB_IMPORTANT,
                          LOC + "Taking too long to be allowed to read..");
                  readErr++;
-                 
+
                  // HACK Sometimes the readhead thread gets borked on startup.
-               /*  if ((readErr % 2) && (rbrpos ==0))
+                 if ((readErr > 4 && readErr % 2) && (rbrpos ==0))
                  {
                     VERBOSE(VB_IMPORTANT, "restarting readhead thread..");
                     KillReadAheadThread();
                     StartupReadAheadThread();
                  }                    
-                   */ 
+ 
                  if (readErr > 10)
                  {
                      VERBOSE(VB_IMPORTANT, LOC_ERR + "Took more than "
@@ -888,13 +986,21 @@ int RingBuffer::ReadFromBuf(void *buf, int count)
         if (!availWait.wait(&availWaitMutex, 250))
         {
             int elapsed = t.elapsed();
-            if (((elapsed > 2000) && (elapsed < 2250)) ||
-                ((elapsed > 4000) && (elapsed < 4250)) ||
-                ((elapsed > 8000) && (elapsed < 8250)))
+            if  (/*((elapsed > 500)  && (elapsed < 750))  ||*/
+                 ((elapsed > 1000) && (elapsed < 1250)) ||
+                 ((elapsed > 2000) && (elapsed < 2250)) ||
+                 ((elapsed > 4000) && (elapsed < 4250)) ||
+                 ((elapsed > 8000) && (elapsed < 8250)))
             {
                 VERBOSE(VB_IMPORTANT, LOC + "Waited " +
-                        QString("%1").arg(elapsed/1000) +
+                        QString("%1").arg((elapsed / 500) * 0.5f, 3, 'f', 1) +
                         " seconds for data to become available...");
+                if (livetvchain)
+                {
+                    VERBOSE(VB_IMPORTANT, "Checking to see if there's a "
+                                          "new livetv program to switch to..");
+                    livetvchain->ReloadAll();
+                }
             }
 
             bool quit = livetvchain && (livetvchain->NeedsToSwitch() || 
@@ -923,7 +1029,7 @@ int RingBuffer::ReadFromBuf(void *buf, int count)
         availWaitMutex.unlock();
 
         avail = ReadBufAvail();
-        if (ateof && avail < count)
+        if ((ateof || setswitchtonext) && avail < count)
             count = avail;
 
         if (commserror)
@@ -944,9 +1050,12 @@ int RingBuffer::ReadFromBuf(void *buf, int count)
     else
         memcpy(buf, readAheadBuffer + rbrpos, count);
 
-    readAheadLock.lock();
-    rbrpos = (rbrpos + count) % kBufferSize;
-    readAheadLock.unlock();
+    if (!peek)
+    {
+        readAheadLock.lock();
+        rbrpos = (rbrpos + count) % kBufferSize;
+        readAheadLock.unlock();
+    }
 
     if (readone)
     {
@@ -984,11 +1093,13 @@ int RingBuffer::Read(void *buf, int count)
             ret = safe_read(remotefile, buf, count);
             readpos += ret;
         }
+#ifdef USING_FRONTEND
         else if (dvdPriv)
         {                        
             ret = dvdPriv->safe_read(buf, count);
             readpos += ret;
         }
+#endif // USING_FRONTEND
         else
         {
             ret = safe_read(fd2, buf, count);
@@ -1072,21 +1183,39 @@ long long RingBuffer::Seek(long long pos, int whence)
     pthread_rwlock_wrlock(&rwlock);
     wantseek = false;
 
+    // optimize nop seeks
+    if ((whence == SEEK_SET && pos == readpos) ||
+        (whence == SEEK_CUR && pos == 0))
+    {
+        pthread_rwlock_unlock(&rwlock);
+        return readpos;
+    }
+
     long long ret = -1;
     if (remotefile)
         ret = remotefile->Seek(pos, whence, readpos);
+#ifdef USING_FRONTEND
     else if (dvdPriv)
     {
-        dvdPriv->Seek(pos, whence);
+        dvdPriv->NormalSeek(pos);
     }
+#endif // USING_FRONTEND
     else
     {
         if (whence == SEEK_SET)
+#ifdef USING_MINGW
+            ret = lseek64(fd2, pos, whence);
+#else
             ret = lseek(fd2, pos, whence);
+#endif
         else
         {
             long long realseek = readpos + pos;
+#ifdef USING_MINGW
+            ret = lseek64(fd2, realseek, SEEK_SET);
+#else
             ret = lseek(fd2, realseek, SEEK_SET);
+#endif
         }
     }
 
@@ -1155,8 +1284,10 @@ void RingBuffer::SetWriteBufferMinWriteSize(int newMinSize)
  */
 long long RingBuffer::GetReadPosition(void) const
 {
+#ifdef USING_FRONTEND
     if (dvdPriv)
         return dvdPriv->GetReadPosition();
+#endif // USING_FRONTEND
 
     return readpos;
 }
@@ -1202,70 +1333,13 @@ void RingBuffer::SetLiveMode(LiveTVChain *chain)
     livetvchain = chain;
 }
 
-/** \fn RingBuffer::getPartAndTitle(int&,int&)
- *  \brief Calls DVDRingBufferPriv::GetPartAndTitle(int&,int&)
- */
-void RingBuffer::getPartAndTitle(int &title, int &part)
+bool RingBuffer::InDVDMenuOrStillFrame(void)
 {
+#ifdef USING_FRONTEND
     if (dvdPriv)
-        dvdPriv->GetPartAndTitle(title, part);
-}
-
-/** \fn RingBuffer::getDescForPos(QString&)
- *  \brief Calls DVDRingBufferPriv::GetDescForPos(QString&)
- */
-void RingBuffer::getDescForPos(QString &desc)
-{
-    if (dvdPriv)
-        dvdPriv->GetDescForPos(desc);
-}
-
-/** \fn RingBuffer::nextTrack(void)
- *  \brief Calls DVDRingBufferPriv::nextTrack(void)
- */
-bool RingBuffer::nextTrack(void)
-{
-    if (dvdPriv)
-        return dvdPriv->nextTrack();
+        return (dvdPriv->IsInMenu() || dvdPriv->InStillFrame());
+#endif // USING_FRONTEND
     return false;
 }
 
-/** \fn RingBuffer::prevTrack(void)
- *  \brief Calls DVDRingBufferPriv::prevTrack(void)
- */
-void RingBuffer::prevTrack(void)
-{
-    if (dvdPriv)
-        dvdPriv->prevTrack();
-}
-
-/** \fn RingBuffer::GetTotalTimeOfTitle(void)
- *  \brief Calls DVDRingBufferPriv::GetTotalTimeOfTitle(void)
- */
-uint RingBuffer::GetTotalTimeOfTitle(void)
-{
-    if (dvdPriv)
-        return dvdPriv->GetTotalTimeOfTitle();
-    return 0;
-}
-
-/** \fn RingBuffer::GetCellStart(void)
- *  \brief Calls DVDRingBufferPriv::GetCellStart(void)
- */
-uint RingBuffer::GetCellStart(void)
-{
-    if (dvdPriv)
-        return dvdPriv->GetCellStart();
-    return 0;
-}
-
-/** \fn RingBuffer::GetTotalReadPosition(void)
- *  \brief Calls DVDRingBufferPriv::GetTotalReadPosition(void)
- */
-long long RingBuffer::GetTotalReadPosition(void)
-{
-    if (dvdPriv)
-        return dvdPriv->GetTotalReadPosition();
-    return 0;
-}
-
+/* vim: set expandtab tabstop=4 shiftwidth=4: */
