@@ -99,8 +99,17 @@ bool HDHRRecorder::Open(void)
         return true;
     }
 
+    /* Calculate buffer size */
+    uint buffersize = gContext->GetNumSetting(
+        "HDRingbufferSize", 50 * TSPacket::SIZE) * 1024;
+    buffersize /= VIDEO_DATA_PACKET_SIZE;
+    buffersize *= VIDEO_DATA_PACKET_SIZE;
+
+    // Buffer should be at least about 1MB..
+    buffersize = max(49 * TSPacket::SIZE * 128, buffersize);
+
     /* Create TS socket. */
-    _video_socket = hdhomerun_video_create(VIDEO_DATA_BUFFER_SIZE_1S, 50);
+    _video_socket = hdhomerun_video_create(0, buffersize);
     if (!_video_socket)
     {
         VERBOSE(VB_IMPORTANT, LOC + "Open() failed to open socket");
@@ -241,9 +250,13 @@ void HDHRRecorder::HandleSingleProgramPMT(ProgramMapTable *pmt)
     if (!pmt)
         return;
 
-    int next = (pmt->tsheader()->ContinuityCounter()+1)&0xf;
-    pmt->tsheader()->SetContinuityCounter(next);
-    BufferedWrite(*(reinterpret_cast<TSPacket*>(pmt->tsheader())));
+    unsigned char buf[8 * 1024];
+    uint next_cc = (pmt->tsheader()->ContinuityCounter()+1)&0xf;
+    pmt->tsheader()->SetContinuityCounter(next_cc);
+    uint size = pmt->WriteAsTSPackets(buf, next_cc);
+
+    for (uint i = 0; i < size ; i += TSPacket::SIZE)
+        DTVRecorder::BufferedWrite(*(reinterpret_cast<TSPacket*>(&buf[i])));
 }
 
 /** \fn HDHRRecorder::HandleMGT(const MasterGuideTable*)
@@ -273,6 +286,13 @@ bool HDHRRecorder::ProcessTSPacket(const TSPacket& tspacket)
         if (tspacket.HasPayload())
         {
             const unsigned int lpid = tspacket.PID();
+
+            if ((GetStreamData()->VideoPIDSingleProgram() > 0x1fff) &&
+                _wait_for_keyframe_option)
+            {
+                _wait_for_keyframe_option = false;
+            }
+
             // Pass or reject frames based on PID, and parse info from them
             if (lpid == GetStreamData()->VideoPIDSingleProgram())
             {
@@ -283,6 +303,7 @@ bool HDHRRecorder::ProcessTSPacket(const TSPacket& tspacket)
             else if (GetStreamData()->IsAudioPID(lpid))
             {
                 //cerr<<"a";
+                _buffer_packets = !FindAudioKeyframes(&tspacket);
                 BufferedWrite(tspacket);
             }
             else if (GetStreamData()->IsListeningPID(lpid))
@@ -341,18 +362,15 @@ void HDHRRecorder::StartRecording(void)
             AdjustFilters();
         }
 
-        unsigned long data_length;
+        size_t read_size = 64 * 1024; // read about 64KB
+        read_size /= VIDEO_DATA_PACKET_SIZE;
+        read_size *= VIDEO_DATA_PACKET_SIZE;
+
+        size_t data_length;
         unsigned char *data_buffer =
-            hdhomerun_video_recv_inplace(_video_socket,
-                                         VIDEO_DATA_BUFFER_SIZE_1S / 5,
-                                         &data_length);
+            hdhomerun_video_recv(_video_socket, read_size, &data_length);
         if (!data_buffer)
         {
-            if (hdhomerun_video_get_state(_video_socket) == 0)
-            {
-                VERBOSE(VB_IMPORTANT, LOC_ERR + "Recv error" + ENO);
-                break;
-            }
             usleep(5000);
 	    continue;
 	}
