@@ -1,10 +1,27 @@
 // -*- Mode: c++ -*-
 // Copyright (c) 2003-2004, Daniel Thor Kristjansson
+
+#include <algorithm> // for find & max
+using namespace std;
+
+// POSIX headers
+#include <sys/time.h> // for gettimeofday
+
+// Qt headers
+#include <qstring.h>
+#include <qdeepcopy.h>
+
+// MythTV headers
 #include "mpegstreamdata.h"
 #include "mpegtables.h"
 #include "RingBuffer.h"
 #include "mpegtables.h"
 
+#include "atscstreamdata.h"
+#include "atsctables.h"
+
+//#define DEBUG_MPEG_RADIO // uncomment to strip video streams from TS stream
+
 void init_sections(sections_t &sect, uint last_section)
 {
     static const unsigned char init_bits[8] =
@@ -15,7 +32,7 @@ void init_sections(sections_t &sect, uint last_section)
     uint endz = last_section >> 3;
     if (endz)
         sect.resize(endz, 0x00);
-    sect.resize(33 - endz, 0xff);
+    sect.resize(32, 0xff);
     sect[endz] = init_bits[last_section & 0x7];
 
 #if 0
@@ -45,11 +62,18 @@ const unsigned char MPEGStreamData::bit_sel[8] =
  *  \param cacheTables    If true PAT and PMT tables will be cached
  */
 MPEGStreamData::MPEGStreamData(int desiredProgram, bool cacheTables)
-    : _have_CRC_bug(false), _eit_helper(NULL), _eit_rate(0.0f),
-      _listener_lock(true),
+    : _sistandard("mpeg"),
+      _have_CRC_bug(false),
+      _local_utc_offset(0), _si_time_offset_cnt(0),
+      _si_time_offset_indx(0),
+      _eit_helper(NULL), _eit_rate(0.0f),
+      _encryption_lock(true), _listener_lock(true),
       _cache_tables(cacheTables), _cache_lock(true),
       // Single program stuff
       _desired_program(desiredProgram),
+      _recording_type("all"),
+      _strip_pmt_descriptors(false),
+      _normalize_stream_type(true),
       _pid_video_single_program(0xffffffff),
       _pid_pmt_single_program(0xffffffff),      
       _pmt_single_program_num_video(1),
@@ -57,6 +81,11 @@ MPEGStreamData::MPEGStreamData(int desiredProgram, bool cacheTables)
       _pat_single_program(NULL), _pmt_single_program(NULL),
       _invalid_pat_seen(false), _invalid_pat_warning(false)
 {
+    _local_utc_offset = calc_utc_offset();
+
+    // MS Windows doesn't like bzero()..
+    memset(_si_time_offsets, 0, sizeof(_si_time_offsets));
+
     AddListeningPID(MPEG_PAT_PID);
 }
 
@@ -109,6 +138,20 @@ void MPEGStreamData::SetDesiredProgram(int p)
         Reset(p);
 }
 
+void MPEGStreamData::SetRecordingType(const QString &recording_type)
+{
+    _recording_type = QDeepCopy<QString>(recording_type);
+    uint neededVideo = (_recording_type == "tv")    ? 1 : 0;
+    uint neededAudio = (_recording_type == "audio") ? 1 : 0;
+    SetVideoStreamsRequired(neededVideo);
+    SetAudioStreamsRequired(neededAudio);
+}
+
+QString MPEGStreamData::GetRecordingType(void) const
+{
+    return QDeepCopy<QString>(_recording_type);
+}
+
 void MPEGStreamData::SetEITHelper(EITHelper *eit_helper)
 {
     QMutexLocker locker(&_listener_lock);
@@ -123,7 +166,11 @@ void MPEGStreamData::SetEITRate(float rate)
 
 void MPEGStreamData::Reset(int desiredProgram)
 {
-    _desired_program = desiredProgram;
+    _desired_program       = desiredProgram;
+    _recording_type        = "all";
+    _strip_pmt_descriptors = false;
+    _normalize_stream_type = true;
+
     _invalid_pat_seen = false;
 
     SetPATSingleProgram(NULL);
@@ -138,6 +185,7 @@ void MPEGStreamData::Reset(int desiredProgram)
     _pids_listening.clear();
     _pids_notlistening.clear();
     _pids_writing.clear();
+    _pids_audio.clear();
 
     _pid_video_single_program = _pid_pmt_single_program = 0xffffffff;
 
@@ -161,6 +209,8 @@ void MPEGStreamData::Reset(int desiredProgram)
         _cached_pmts.clear();
     }
 
+    ResetDecryptionMonitoringState();
+
     AddListeningPID(MPEG_PAT_PID);
 }
 
@@ -237,6 +287,11 @@ PSIPTable* MPEGStreamData::AssemblePSIP(const TSPacket* tspacket,
         {
             if (partial->pesdata()[psip->SectionLength()] != 0xff)
             {
+#if 0 /* This doesn't work, you can't start PES packet like this 
+         because the PayloadStart() flag won't be set in this TSPacket
+         -- dtk  May 4th, 2007
+       */
+
                 // If the next section starts in the new tspacket
                 // create a new partial packet to prevent overflow
                 if ((partial->TSSizeInBuffer() > TSPacket::SIZE) &&
@@ -248,6 +303,7 @@ PSIPTable* MPEGStreamData::AssemblePSIP(const TSPacket* tspacket,
                                    new PESPacket(*tspacket));
                 }
                 else
+#endif
                 {
                     partial->SetPSIOffset(partial->PSIOffset() +
                                           psip->SectionLength());
@@ -392,7 +448,51 @@ bool MPEGStreamData::CreatePATSingleProgram(
 
 }
 
-bool MPEGStreamData::CreatePMTSingleProgram(const ProgramMapTable& pmt)
+desc_list_t extract_atsc_desc(const tvct_vec_t &tvct,
+                              const cvct_vec_t &cvct,
+                              uint pnum)
+{
+    desc_list_t desc;
+
+    vector<const VirtualChannelTable*> vct;
+
+    for (uint i = 0; i < tvct.size(); i++)
+        vct.push_back(tvct[i]);
+
+    for (uint i = 0; i < cvct.size(); i++)
+        vct.push_back(cvct[i]);
+
+    for (uint i = 0; i < tvct.size(); i++)
+    {
+        for (uint j = 0; j < vct[i]->ChannelCount(); j++)
+        {
+            if (vct[i]->ProgramNumber(j) == pnum)
+            {
+                desc_list_t ldesc = MPEGDescriptor::ParseOnlyInclude(
+                    vct[i]->Descriptors(j), vct[i]->DescriptorsLength(j),
+                    DescriptorID::caption_service);
+
+                if (ldesc.size())
+                    desc.insert(desc.end(), ldesc.begin(), ldesc.end());
+            }
+        }
+
+        if (0 != vct[i]->GlobalDescriptorsLength())
+        {
+            desc_list_t vdesc = MPEGDescriptor::ParseOnlyInclude(
+                vct[i]->GlobalDescriptors(),
+                vct[i]->GlobalDescriptorsLength(),
+                DescriptorID::caption_service); 
+
+            if (vdesc.size())
+                desc.insert(desc.end(), vdesc.begin(), vdesc.end());
+        }
+    }
+
+    return desc;
+}
+
+bool MPEGStreamData::CreatePMTSingleProgram(const ProgramMapTable &pmt)
 {
     VERBOSE(VB_RECORD, "CreatePMTSingleProgram()");
     VERBOSE(VB_RECORD, "PMT in input stream");
@@ -405,34 +505,102 @@ bool MPEGStreamData::CreatePMTSingleProgram(const ProgramMapTable& pmt)
     }
     pmt.Parse();
 
-    vector<uint> videoPIDs, audioPIDs;
-    vector<uint> videoTypes;
-    vector<uint> pids, types;
+    uint programNumber = 1; // MPEG Program Number
+
+    ATSCStreamData *sd = NULL;
+    tvct_vec_t tvct;
+    cvct_vec_t cvct;
+
+    desc_list_t gdesc;
+
+    if (!_strip_pmt_descriptors)
+    {
+        gdesc = MPEGDescriptor::ParseAndExclude(
+            pmt.ProgramInfo(), pmt.ProgramInfoLength(),
+            DescriptorID::conditional_access);
+
+        // If there is no caption descriptor in PMT, copy any caption
+        // descriptor found in VCT to global descriptors...
+        sd = dynamic_cast<ATSCStreamData*>(this);
+        if (sd && !MPEGDescriptor::Find(gdesc, DescriptorID::caption_service))
+        {
+            tvct = sd->GetAllCachedTVCTs();
+            cvct = sd->GetAllCachedCVCTs();
+
+            desc_list_t vdesc = extract_atsc_desc(
+                tvct, cvct, pmt.ProgramNumber());
+
+            if (vdesc.size())
+                gdesc.insert(gdesc.end(), vdesc.begin(), vdesc.end());
+        }
+    }
+
+    vector<uint> pids;
+    vector<uint> types;
+    vector<desc_list_t> pdesc;
+
+    uint video_cnt = 0;
+    uint audio_cnt = 0;
+
+    vector<uint> videoPIDs, audioPIDs, dataPIDs;
+
+    for (uint i = 0; i < pmt.StreamCount(); i++)
+    {
+        desc_list_t desc = MPEGDescriptor::ParseAndExclude(
+            pmt.StreamInfo(i), pmt.StreamInfoLength(i),
+            DescriptorID::conditional_access);
+
+        uint type = StreamID::Normalize(
+            pmt.StreamType(i), desc, _sistandard);
+
+        bool is_video = StreamID::IsVideo(type);
+        bool is_audio = StreamID::IsAudio(type);
+
+        uint pid = pmt.StreamPID(i);
+
+        if (is_audio)
+        {
+            audio_cnt++;
+            audioPIDs.push_back(pid);
+        }
+
+#ifdef DEBUG_MPEG_RADIO
+        if (is_video)
+            continue;
+#endif // DEBUG_MPEG_RADIO
+
+        if (is_video)
+        {
+            video_cnt++;
+            videoPIDs.push_back(pid);
+        }
+
+        if (_strip_pmt_descriptors)
+            desc.clear();
+
+        // Filter out streams not used for basic television
+        if (_recording_type == "tv" && !is_audio && !is_video &&
+            !MPEGDescriptor::Find(desc, DescriptorID::teletext) &&
+            !MPEGDescriptor::Find(desc, DescriptorID::subtitling))
+        {
+            continue;
+        }
+
+        if (!is_audio && !is_video)
+            dataPIDs.push_back(pid);
+
+        pdesc.push_back(desc);
+        pids.push_back(pid);
+        types.push_back(type);
+    }
 
-    // Video
-    uint video_cnt = pmt.FindPIDs(StreamID::AnyVideo, videoPIDs, videoTypes, true);
     if (video_cnt < _pmt_single_program_num_video) 
     {
         VERBOSE(VB_RECORD, "Only "<<video_cnt<<" video streams seen in PMT, "
                 "but "<<_pmt_single_program_num_video<<" are required.");
         return false;
     }
-    if (videoPIDs.size() > 1)
-    {
-        VERBOSE(VB_RECORD,
-                "Warning: More than one video stream in old PMT, "
-                "only using first one in new PMT");
-    }
 
-    if (videoPIDs.size() > 0)
-    {
-        _pid_video_single_program = videoPIDs[0];
-        pids.push_back(videoPIDs[0]);
-        types.push_back(videoTypes[0]);
-    }
-
-    // Audio
-    pmt.FindPIDs(StreamID::AnyAudio, audioPIDs);
     if (audioPIDs.size() < _pmt_single_program_num_audio)
     {
         VERBOSE(VB_RECORD, "Only "<<audioPIDs.size()
@@ -440,9 +608,19 @@ bool MPEGStreamData::CreatePMTSingleProgram(const ProgramMapTable& pmt)
                 <<_pmt_single_program_num_audio<<" are required.");
         return false;
     }
+
+    _pids_audio.clear();
     for (uint i = 0; i < audioPIDs.size(); i++)
         AddAudioPID(audioPIDs[i]);
 
+    if (videoPIDs.size() >= 1)
+        _pid_video_single_program = videoPIDs[0];
+    for (uint i = 1; i < videoPIDs.size(); i++)
+        AddWritingPID(videoPIDs[i]);
+
+    for (uint i = 0; i < dataPIDs.size(); i++)
+        AddWritingPID(dataPIDs[i]);
+
     // Timebase
     int pcrpidIndex = pmt.FindPID(pmt.PCRPID());
     if (pcrpidIndex < 0)
@@ -452,14 +630,17 @@ bool MPEGStreamData::CreatePMTSingleProgram(const ProgramMapTable& pmt)
         AddWritingPID(pmt.PCRPID());
     }
 
-    // Misc
-    uint programNumber = 1;
+    // Create the PMT
+    ProgramMapTable *pmt2 = ProgramMapTable::Create(
+        programNumber, _pid_pmt_single_program, pmt.PCRPID(),
+        pmt.Version(), gdesc, pids, types, pdesc);
 
-    // Construct
-    pmt.FindPIDs(StreamID::AnyAudio, pids, types, true);
-    ProgramMapTable *pmt2 = ProgramMapTable::
-        Create(programNumber, _pid_pmt_single_program,
-               pmt.PCRPID(), pmt.Version(), pids, types);
+    // Return any TVCT & CVCT tables, once we've copied any descriptors.
+    if (sd)
+    {
+        sd->ReturnCachedTVCTTables(tvct);
+        sd->ReturnCachedCVCTTables(cvct);
+    }
 
     // Set Continuity Header
     uint cc_cnt = pmt.tsheader()->ContinuityCounter();
@@ -625,6 +806,39 @@ void MPEGStreamData::ProcessPMT(const ProgramMapTable *pmt)
     }
 }
 
+double MPEGStreamData::TimeOffset(void) const
+{
+    QMutex locker(&_si_time_lock);
+    if (!_si_time_offset_cnt)
+        return 0.0;
+
+    double avg_offset = 0.0;
+    double mult = 1.0 / _si_time_offset_cnt;
+    for (uint i = 0; i < _si_time_offset_cnt; i++)
+        avg_offset += _si_time_offsets[i] * mult;
+
+    return avg_offset;
+}
+
+void MPEGStreamData::UpdateTimeOffset(uint64_t _si_utc_time)
+{
+    struct timeval tm;
+    if (gettimeofday(&tm, NULL) != 0)
+        return;
+
+    double utc_time = tm.tv_sec + (tm.tv_usec * 0.000001);
+    double si_time  = _si_utc_time;
+
+    QMutex locker(&_si_time_lock);
+    _si_time_offsets[_si_time_offset_indx] = si_time - utc_time;
+
+    if (_si_time_offset_indx + 1 > _si_time_offset_cnt)
+        _si_time_offset_cnt = _si_time_offset_indx + 1;
+
+    _si_time_offset_indx = (_si_time_offset_indx + 1) & 0xf;
+
+}
+
 #define DONE_WITH_PES_PACKET() { if (psip) delete psip; \
     if (morePSIPPackets) goto HAS_ANOTHER_PES; else return; }
 
@@ -640,6 +854,13 @@ void MPEGStreamData::HandleTSTables(const TSPacket* tspacket)
     if (!psip)
        return;
 
+    // Don't do the usual checks on TDT - it has no CRC, etc...
+    if (TableID::TDT == psip->TableID())
+    {
+        HandleTables(tspacket->PID(), *psip);
+        DONE_WITH_PES_PACKET();
+    }
+
     // drop stuffing packets
     if ((TableID::ST       == psip->TableID()) ||
         (TableID::STUFFING == psip->TableID()))
@@ -668,6 +889,12 @@ void MPEGStreamData::HandleTSTables(const TSPacket* tspacket)
         DONE_WITH_PES_PACKET();
     }
 
+    if (!psip->VerifyPSIP(!_have_CRC_bug))
+    {
+        VERBOSE(VB_RECORD, "PSIP table is invalid");
+        DONE_WITH_PES_PACKET();
+    }
+
     // Don't decode redundant packets,
     // but if it is a desired PAT or PMT emit a "heartbeat" signal.
     if (IsRedundant(tspacket->PID(), *psip))
@@ -691,19 +918,21 @@ void MPEGStreamData::HandleTSTables(const TSPacket* tspacket)
     }
 
     HandleTables(tspacket->PID(), *psip);
+
     DONE_WITH_PES_PACKET();
 }
 #undef DONE_WITH_PES_PACKET
 
-int MPEGStreamData::ProcessData(unsigned char *buffer, int len)
+int MPEGStreamData::ProcessData(const unsigned char *buffer, int len)
 {
     int pos = 0;
+    bool resync = false;
 
     while (pos + 187 < len) // while we have a whole packet left
     {
-        if (buffer[pos] != SYNC_BYTE)
+        if (buffer[pos] != SYNC_BYTE || resync)
         {
-            int newpos = ResyncStream(buffer, pos, len);
+            int newpos = ResyncStream(buffer, pos+1, len);
             if (newpos == -1)
                 return len - pos;
             if (newpos == -2)
@@ -714,9 +943,12 @@ int MPEGStreamData::ProcessData(unsigned char *buffer, int len)
 
         const TSPacket *pkt = reinterpret_cast<const TSPacket*>(&buffer[pos]);
         if (ProcessTSPacket(*pkt))
+        {
             pos += TSPacket::SIZE; // Advance to next TS packet
+            resync = false;
+        }
         else // Let it resync in case of dropped bytes
-            buffer[pos] = SYNC_BYTE + 1;
+            resync = true;
     }
 
     return len - pos;
@@ -725,15 +957,50 @@ int MPEGStreamData::ProcessData(unsigned char *buffer, int len)
 bool MPEGStreamData::ProcessTSPacket(const TSPacket& tspacket)
 {
     bool ok = !tspacket.TransportError();
-    if (ok && !tspacket.ScramplingControl() && tspacket.HasPayload() &&
-        IsListeningPID(tspacket.PID()))
+
+    if (IsEncryptionTestPID(tspacket.PID()))
     {
-        HandleTSTables(&tspacket);
+        ProcessEncryptedPacket(tspacket);
     }
-    return ok;
+
+    if (!ok)
+        return false;
+
+    if (!tspacket.ScramplingControl() && tspacket.HasPayload())
+    {
+        if (IsVideoPID(tspacket.PID()))
+        {
+            for (uint j = 0; j < _ts_av_listeners.size(); j++)
+                _ts_av_listeners[j]->ProcessVideoTSPacket(tspacket);
+
+            return true;
+        }
+
+        if (IsAudioPID(tspacket.PID()))
+        {
+            for (uint j = 0; j < _ts_av_listeners.size(); j++)
+                _ts_av_listeners[j]->ProcessAudioTSPacket(tspacket);
+
+            return true;
+        }
+
+        if (IsWritingPID(tspacket.PID()) && _ts_writing_listeners.size())
+        {
+            for (uint j = 0; j < _ts_writing_listeners.size(); j++)
+                _ts_writing_listeners[j]->ProcessTSPacket(tspacket);
+        }
+
+        if (IsListeningPID(tspacket.PID()))
+        {
+            HandleTSTables(&tspacket);
+        }
+    }
+
+    return true;
 }
 
-int MPEGStreamData::ResyncStream(unsigned char *buffer, int curr_pos, int len)
+int MPEGStreamData::ResyncStream(const unsigned char *buffer, int curr_pos,
+                                 int len)
 {
     // Search for two sync bytes 188 bytes apart, 
     int pos = curr_pos;
@@ -754,28 +1021,72 @@ int MPEGStreamData::ResyncStream(unsigned char *buffer, int curr_pos, int len)
 
 bool MPEGStreamData::IsListeningPID(uint pid) const
 {
-    QMap<uint, bool>::const_iterator it = _pids_listening.find(pid);
+    pid_map_t::const_iterator it = _pids_listening.find(pid);
     return it != _pids_listening.end();
 }
 
 bool MPEGStreamData::IsNotListeningPID(uint pid) const
 {
-    QMap<uint, bool>::const_iterator it = _pids_notlistening.find(pid);
+    pid_map_t::const_iterator it = _pids_notlistening.find(pid);
     return it != _pids_notlistening.end();
 }
 
 bool MPEGStreamData::IsWritingPID(uint pid) const
 {
-    QMap<uint, bool>::const_iterator it = _pids_writing.find(pid);
+    pid_map_t::const_iterator it = _pids_writing.find(pid);
     return it != _pids_writing.end();
 }
 
 bool MPEGStreamData::IsAudioPID(uint pid) const
 {
-    QMap<uint, bool>::const_iterator it = _pids_audio.find(pid);
+    pid_map_t::const_iterator it = _pids_audio.find(pid);
     return it != _pids_audio.end();
 }
 
+uint MPEGStreamData::GetPIDs(pid_map_t &pids) const
+{
+    uint sz = pids.size();
+
+    if (_pid_video_single_program < 0x1fff)
+        pids[_pid_video_single_program] = kPIDPriorityHigh;
+
+    pid_map_t::const_iterator it = _pids_listening.begin();
+    for (; it != _pids_listening.end(); ++it)
+        pids[it.key()] = max(pids[it.key()], *it);
+
+    it = _pids_audio.begin();
+    for (; it != _pids_audio.end(); ++it)
+        pids[it.key()] = max(pids[it.key()], *it);
+
+    it = _pids_writing.begin();
+    for (; it != _pids_writing.end(); ++it)
+        pids[it.key()] = max(pids[it.key()], *it);
+
+    return pids.size() - sz;
+}
+
+PIDPriority MPEGStreamData::GetPIDPriority(uint pid) const
+{
+    if (_pid_video_single_program == pid)
+        return kPIDPriorityHigh;
+
+    pid_map_t::const_iterator it;
+    it = _pids_listening.find(pid);
+    if (it != _pids_listening.end())
+        return *it;
+    it = _pids_notlistening.find(pid);
+    if (it != _pids_notlistening.end())
+        return *it;
+    it = _pids_writing.find(pid);
+    if (it != _pids_writing.end())
+        return *it;
+    it = _pids_audio.find(pid);
+    if (it != _pids_audio.end())
+        return *it;
+
+    return kPIDPriorityNone;
+}
+
 void MPEGStreamData::SavePartialPES(uint pid, PESPacket* packet)
 {
     pid_pes_map_t::iterator it = _partial_pes_packet_cache.find(pid);
@@ -1161,6 +1472,60 @@ void MPEGStreamData::RemoveMPEGListener(MPEGStreamListener *val)
     }
 }
 
+void MPEGStreamData::AddWritingListener(TSPacketListener *val)
+{
+    QMutexLocker locker(&_listener_lock);
+
+    ts_listener_vec_t::iterator it = _ts_writing_listeners.begin();
+    for (; it != _ts_writing_listeners.end(); ++it)
+        if (((void*)val) == ((void*)*it))
+            return;
+
+    _ts_writing_listeners.push_back(val);
+}
+
+void MPEGStreamData::RemoveWritingListener(TSPacketListener *val)
+{
+    QMutexLocker locker(&_listener_lock);
+
+    ts_listener_vec_t::iterator it = _ts_writing_listeners.begin();
+    for (; it != _ts_writing_listeners.end(); ++it)
+    {
+        if (((void*)val) == ((void*)*it))
+        {
+            _ts_writing_listeners.erase(it);
+            return;
+        }
+    }
+}
+
+void MPEGStreamData::AddAVListener(TSPacketListenerAV *val)
+{
+    QMutexLocker locker(&_listener_lock);
+
+    ts_av_listener_vec_t::iterator it = _ts_av_listeners.begin();
+    for (; it != _ts_av_listeners.end(); ++it)
+        if (((void*)val) == ((void*)*it))
+            return;
+
+    _ts_av_listeners.push_back(val);
+}
+
+void MPEGStreamData::RemoveAVListener(TSPacketListenerAV *val)
+{
+    QMutexLocker locker(&_listener_lock);
+
+    ts_av_listener_vec_t::iterator it = _ts_av_listeners.begin();
+    for (; it != _ts_av_listeners.end(); ++it)
+    {
+        if (((void*)val) == ((void*)*it))
+        {
+            _ts_av_listeners.erase(it);
+            return;
+        }
+    }
+}
+
 void MPEGStreamData::AddMPEGSPListener(MPEGSingleProgramStreamListener *val)
 {
     QMutexLocker locker(&_listener_lock);
@@ -1187,3 +1552,211 @@ void MPEGStreamData::RemoveMPEGSPListener(MPEGSingleProgramStreamListener *val)
         }
     }
 }
+
+void MPEGStreamData::AddEncryptionTestPID(uint pnum, uint pid, bool isvideo)
+{
+    QMutexLocker locker(&_encryption_lock);
+
+    //VERBOSE(VB_IMPORTANT, "AddEncryptionTestPID("<<pnum
+    //        <<", 0x"<<hex<<pid<<dec<<")");
+
+    AddListeningPID(pid);
+
+    _encryption_pid_to_info[pid] = CryptInfo((isvideo) ? 10000 : 500, 8);
+    
+    _encryption_pid_to_pnums[pid].push_back(pnum);
+    _encryption_pnum_to_pids[pnum].push_back(pid);
+    _encryption_pnum_to_status[pnum] = kEncUnknown;
+}
+
+void MPEGStreamData::RemoveEncryptionTestPIDs(uint pnum)
+{
+    QMutexLocker locker(&_encryption_lock);
+
+    //VERBOSE(VB_RECORD,
+    //        QString("Tearing down up decryption monitoring "
+    //                "for program %1").arg(pnum));
+
+    QMap<uint, uint_vec_t>::iterator list;
+    uint_vec_t::iterator it;
+
+    uint_vec_t pids = _encryption_pnum_to_pids[pnum];
+    for (uint i = 0; i < pids.size(); i++)
+    {
+        uint pid = pids[i];
+
+        //VERBOSE(VB_IMPORTANT, QString("Removing 0x%1 PID Enc monitoring")
+        //        .arg(pid,0,16));
+
+        RemoveListeningPID(pid);
+
+        list = _encryption_pid_to_pnums.find(pid);
+        if (list != _encryption_pid_to_pnums.end())
+        {
+            it = find((*list).begin(), (*list).end(), pnum);
+
+            if (it != (*list).end())
+                (*list).erase(it);
+
+            if ((*list).empty())
+            {
+                _encryption_pid_to_pnums.erase(pid);
+                _encryption_pid_to_info.erase(pid);
+            }
+        }
+    }
+
+    _encryption_pnum_to_pids.erase(pnum);
+}
+
+bool MPEGStreamData::IsEncryptionTestPID(uint pid) const
+{
+    QMutexLocker locker(&_encryption_lock);
+
+    QMap<uint, CryptInfo>::const_iterator it =
+        _encryption_pid_to_info.find(pid);
+
+    return it != _encryption_pid_to_info.end();
+}
+
+void MPEGStreamData::TestDecryption(const ProgramMapTable *pmt)
+{
+    QMutexLocker locker(&_encryption_lock);
+
+    //VERBOSE(VB_RECORD,
+    //        QString("Setting up decryption monitoring "
+    //                "for program %1").arg(pmt->ProgramNumber()));
+
+    bool encrypted = pmt->IsProgramEncrypted();
+    for (uint i = 0; i < pmt->StreamCount(); i++)
+    {
+        if (!encrypted && !pmt->IsStreamEncrypted(i))
+            continue;
+
+        bool is_vid = pmt->IsVideo(i, _sistandard);
+        bool is_aud = pmt->IsAudio(i, _sistandard);
+        if (is_vid || is_aud)
+        {
+            AddEncryptionTestPID(
+                pmt->ProgramNumber(), pmt->StreamPID(i), is_vid);
+        }
+    }
+}
+
+void MPEGStreamData::ResetDecryptionMonitoringState(void)
+{
+    QMutexLocker locker(&_encryption_lock);
+
+    _encryption_pid_to_info.clear();
+    _encryption_pid_to_pnums.clear();
+    _encryption_pnum_to_pids.clear();
+}
+
+bool MPEGStreamData::IsProgramDecrypted(uint pnum) const
+{
+    QMutexLocker locker(&_encryption_lock);
+    return _encryption_pnum_to_status[pnum] == kEncDecrypted;
+}
+
+bool MPEGStreamData::IsProgramEncrypted(uint pnum) const
+{
+    QMutexLocker locker(&_encryption_lock);
+    return _encryption_pnum_to_status[pnum] == kEncEncrypted;
+}
+
+static QString toString(CryptStatus status)
+{
+    if (kEncDecrypted == status)
+        return "Decrypted";
+    else if (kEncEncrypted == status)
+        return "Encrypted";
+    else
+        return "Unknown";
+}
+
+/** \fn MPEGStreamData::ProcessEncryptedPacket(const TSPacket& tspacket)
+ *  \brief counts en/decrypted packets to decide if a stream is en/decrypted
+ */
+void MPEGStreamData::ProcessEncryptedPacket(const TSPacket& tspacket)
+{
+    QMutexLocker locker(&_encryption_lock);
+
+    const uint pid = tspacket.PID();
+    CryptInfo &info = _encryption_pid_to_info[pid];
+
+    CryptStatus status = kEncUnknown;
+
+    if (tspacket.ScramplingControl())
+    {
+        info.decrypted_packets = 0;
+
+        // If a fair amount of encrypted packets is passed assume that
+        // the stream is not decryptable
+        if (++info.encrypted_packets >= info.encrypted_min)
+            status = kEncEncrypted;
+    }
+    else
+    {
+        info.encrypted_packets = 0;
+        if (++info.decrypted_packets > info.decrypted_min)
+            status = kEncDecrypted;
+    }
+
+    if (status == info.status)
+        return; // pid encryption status unchanged
+
+    info.status = status;
+
+    VERBOSE(status != kEncDecrypted ? VB_IMPORTANT : VB_RECORD,
+            QString("PID 0x%1 status: %2")
+            .arg(pid,0,16).arg(toString(status)));
+
+    uint_vec_t pnum_del_list;
+    const uint_vec_t &pnums = _encryption_pid_to_pnums[pid];
+    for (uint i = 0; i < pnums.size(); i++)
+    {
+        CryptStatus status = _encryption_pnum_to_status[pnums[i]];
+
+        const uint_vec_t &pids = _encryption_pnum_to_pids[pnums[i]];
+        if (!pids.empty())
+        {
+            uint enc_cnt[3] = { 0, 0, 0 };
+            for (uint j = 0; j < pids.size(); j++)
+            {
+                CryptStatus stat = _encryption_pid_to_info[pids[j]].status;
+                enc_cnt[stat]++;
+
+                //VERBOSE(VB_IMPORTANT,
+                //        QString("\tpnum %1 PID 0x%2 status: %3")
+                //        .arg(pnums[i]).arg(pids[j],0,16)
+                //        .arg(toString(stat)));
+            }
+            status = kEncUnknown;
+
+            if (enc_cnt[kEncEncrypted])
+                status = kEncEncrypted;
+            else if (enc_cnt[kEncDecrypted] >= min((size_t) 2, pids.size()))
+                status = kEncDecrypted;
+        }
+
+        if (status == _encryption_pnum_to_status[pnums[i]])
+            continue; // program encryption status unchanged
+
+        VERBOSE(VB_RECORD, QString("Program %1 status: %2")
+                .arg(pnums[i]).arg(toString(status)));
+
+        _encryption_pnum_to_status[pnums[i]] = status;
+
+        bool encrypted = kEncUnknown == status || kEncEncrypted == status;
+        _listener_lock.lock();
+        for (uint j = 0; j < _mpeg_listeners.size(); j++)
+            _mpeg_listeners[j]->HandleEncryptionStatus(pnums[i], encrypted);
+        _listener_lock.unlock();
+
+        if (kEncDecrypted == status)
+            pnum_del_list.push_back(pnums[i]);
+    }
+
+    for (uint i = 0; i < pnum_del_list.size(); i++)
+        RemoveEncryptionTestPIDs(pnums[i]);
+}
