@@ -19,9 +19,12 @@ class ProgramRecPriorityInfo : public ProgramInfo
     ProgramRecPriorityInfo& operator=(const ProgramInfo&);
 
 
-    int channelRecPriority;
     int recTypeRecPriority;
     RecordingType recType;
+    int matchCount;
+    int recCount;
+    QDateTime last_record;
+    int avg_delay;
 };
 
 class ProgramRecPriority : public MythDialog
@@ -32,7 +35,11 @@ class ProgramRecPriority : public MythDialog
     {
         byTitle,
         byRecPriority,
-        byRecType
+        byRecType,
+        byCount,
+        byRecCount,
+        byLastRecord,
+        byAvgDelay
     };
 
     ProgramRecPriority(MythMainWindow *parent, const char *name = 0);
@@ -46,8 +53,11 @@ class ProgramRecPriority : public MythDialog
     void changeRecPriority(int howMuch);
     void saveRecPriority(void);
     void edit();
+    void customEdit();
+    void remove();
     void deactivate();
     void upcoming();
+    void details();
 
   protected:
     void paintEvent(QPaintEvent *);
@@ -55,9 +65,12 @@ class ProgramRecPriority : public MythDialog
 
   private:
     void FillList(void);
-    void SortList();
+    void SortList(void);
+    void RemoveCurItemFromList(void);
     QMap<QString, ProgramRecPriorityInfo> programData;
     QMap<int, int> origRecPriorityData;
+
+    void countMatches(void);
     QMap<int, int> conMatch;
     QMap<int, int> nowMatch;
     QMap<int, int> recMatch;
@@ -72,9 +85,6 @@ class ProgramRecPriority : public MythDialog
     XMLParse *theme;
     QDomElement xmldata;
 
-    QString channelFormat;
-    QString longChannelFormat;
-
     ProgramRecPriorityInfo *curitem;
 
     QPixmap myBackground;
