@@ -3,14 +3,18 @@
 #include <qtextstream.h>
 
 #include <iostream>
+#include <cstdlib>
 using namespace std;
 #include <unistd.h>
 
-#include <sys/wait.h>   // For WIFEXITED on Mac OS X
-
 #include <exitcodes.h>
 #include <mythcontext.h>
 #include <mythdbcon.h>
+#include "libmythtv/programinfo.h"
+#include "libmythtv/jobqueue.h"
+#include "tv.h"
+#include "remoteutil.h"
+#include "compat.h"
 
 void setGlobalSetting(const QString &key, const QString &value)
 {
@@ -70,7 +74,42 @@ int lockShutdown()
 {
     VERBOSE(VB_GENERAL, "Mythshutdown: --lock");
 
-    setGlobalSetting("MythShutdownLock", "1");
+    MSqlQuery query(MSqlQuery::InitCon());
+
+    // lock setting table
+    int tries = 0;
+    while (!query.exec("LOCK TABLE settings WRITE;") && tries < 5)
+    {
+        VERBOSE(VB_GENERAL, "Waiting for lock on setting table");
+        sleep(1);
+        tries++;
+    }
+
+    if (tries >= 5)
+    {
+        VERBOSE(VB_GENERAL, "Waited too long to obtain lock on setting table");
+        return 1;
+    }
+
+    // does the setting already exist?
+    query.exec("SELECT * FROM settings "
+               "WHERE value = 'MythShutdownLock' AND hostname IS NULL;");
+
+    if (query.size() < 1)
+    {
+        // add the lock setting
+        query.exec("INSERT INTO settings (value, data) "
+                   "VALUES ('MythShutdownLock', '1');");
+    }
+    else
+    {
+        // update the lock setting
+        query.exec("UPDATE settings SET data = data + 1 "
+                   "WHERE value = 'MythShutdownLock' AND hostname IS NULL;");
+    }
+
+    // unlock settings table
+    query.exec("UNLOCK TABLES;");
 
     return 0;
 }
@@ -79,7 +118,45 @@ int unlockShutdown()
 {
     VERBOSE(VB_GENERAL, "Mythshutdown: --unlock");
 
-    setGlobalSetting("MythShutdownLock", "0");
+    MSqlQuery query(MSqlQuery::InitCon());
+
+    // lock setting table
+    int tries = 0;
+    while (!query.exec("LOCK TABLE settings WRITE;") && tries < 5)
+    {
+        VERBOSE(VB_GENERAL, "Waiting for lock on setting table");
+        sleep(1);
+        tries++;
+    }
+
+    if (tries >= 5)
+    {
+        VERBOSE(VB_GENERAL, "Waited too long to obtain lock on setting table");
+        return 1;
+    }
+
+    // does the setting exist?
+    query.exec("SELECT * FROM settings "
+               "WHERE value = 'MythShutdownLock' AND hostname IS NULL;");
+
+    if (query.size() < 1)
+    {
+        // add the lock setting
+        query.exec("INSERT INTO settings (value, data) "
+                   "VALUES ('MythShutdownLock', '0');");
+    }
+    else
+    {
+        // update lock setting
+        query.exec("UPDATE settings SET data = GREATEST(0,  data - 1) "
+                   "WHERE value = 'MythShutdownLock' AND hostname IS NULL;");
+    }
+
+    // unlock table
+    query.exec("UNLOCK TABLES;");
+
+    // tell the master BE to reset its idle time
+    RemoteSendMessage("RESET_IDLETIME");
 
     return 0;
 }
@@ -90,8 +167,8 @@ bool isRunning(QString program)
     int res = system(sCommand.ascii());
     if (WIFEXITED(res))
         res = WEXITSTATUS(res);
-    
-    return res;    
+
+    return res;
 }
 
 QDateTime getDailyWakeupTime(QString sPeriod)
@@ -103,42 +180,86 @@ QDateTime getDailyWakeupTime(QString sPeriod)
     return dtDateTime;
 }
 
-int getStatus()
+bool isRecording()
+{
+    if (!gContext->IsConnectedToMaster())
+    {
+        VERBOSE(VB_IMPORTANT, "isRecording: Attempting to connect to master server...");
+        if (!gContext->ConnectToMasterServer(false))
+        {
+            VERBOSE(VB_IMPORTANT, "isRecording: Could not connect to master server!");
+            return false;
+        }
+    }
+
+    return RemoteGetRecordingStatus(NULL, false);
+}
+
+int getStatus(bool bWantRecStatus)
 {
     VERBOSE(VB_GENERAL, "Mythshutdown: --status");
-    
-    int res = 0;         
-    
+
+    int res = 0;
+
     if (isRunning("mythtranscode"))
-    {    
+    {
         VERBOSE(VB_IMPORTANT, "Transcoding in progress...");
         res += 1;
-    }    
-        
+    }
+
     if (isRunning("mythcommflag"))
-    {    
+    {
         VERBOSE(VB_IMPORTANT, "Commercial Flagging in progress...");
         res += 2;
-    }    
+    }
 
     if (isRunning("mythfilldatabas"))
-    {    
+    {
         VERBOSE(VB_IMPORTANT, "Grabbing EPG data in progress...");
         res += 4;
-    }    
+    }
 
-    if (getGlobalSetting("MythShutdownLock", "0") == "1")
-    {    
+    if (bWantRecStatus && isRecording())
+    {
+        VERBOSE(VB_IMPORTANT, "Recording in progress...");
+        res += 8;
+    }
+
+    if (getGlobalSetting("MythShutdownLock", "0") != "0")
+    {
         VERBOSE(VB_IMPORTANT, "Shutdown is locked");
         res += 16;
-    }    
-    
+    }
+
+    if (JobQueue::HasRunningOrPendingJobs(15))
+    {
+        VERBOSE(VB_IMPORTANT, "Has queued or pending jobs");
+        res += 32;
+    }
+
     QDateTime dtPeriod1Start = getDailyWakeupTime("DailyWakeupStartPeriod1");
     QDateTime dtPeriod1End = getDailyWakeupTime("DailyWakeupEndPeriod1");
     QDateTime dtPeriod2Start = getDailyWakeupTime("DailyWakeupStartPeriod2");
     QDateTime dtPeriod2End = getDailyWakeupTime("DailyWakeupEndPeriod2");
     QDateTime dtCurrent = QDateTime::currentDateTime();
-    
+
+    // Check for time periods that cross midnight
+    if (dtPeriod1End < dtPeriod1Start)
+    {
+        if (dtCurrent > dtPeriod1End)
+            dtPeriod1End = dtPeriod1End.addDays(1);
+        else
+            dtPeriod1Start = dtPeriod1Start.addDays(-1);
+    }
+
+    if (dtPeriod2End < dtPeriod2Start)
+    {
+        if (dtCurrent > dtPeriod2End)
+            dtPeriod2End = dtPeriod2End.addDays(1);
+        else
+            dtPeriod2Start = dtPeriod2Start.addDays(-1);
+    }
+
     // Check for one of the daily wakeup periods
     if (dtPeriod1Start != dtPeriod1End)
     {
@@ -146,16 +267,16 @@ int getStatus()
         {
             VERBOSE(VB_IMPORTANT, "In a daily wakeup period (1).");
             res |= 64;
-        } 
+        }
     }
-    
+
     if (dtPeriod2Start != dtPeriod2End)
     {
         if (dtCurrent >= dtPeriod2Start && dtCurrent <= dtPeriod2End)
         {
             VERBOSE(VB_IMPORTANT, "In a daily wakeup period (2).");
             res |= 64;
-        } 
+        }
     }
 
     // Are we about to start a daily wakeup period
@@ -167,9 +288,9 @@ int getStatus()
         {
             VERBOSE(VB_IMPORTANT, "About to start daily wakeup period (1)");
             res |= 128;
-        } 
+        }
     }
-    
+
     if (dtPeriod2Start != dtPeriod2End)
     {
         int delta = dtCurrent.secsTo(dtPeriod2Start);
@@ -177,33 +298,39 @@ int getStatus()
         {
             VERBOSE(VB_IMPORTANT, "About to start daily wakeup period (2)");
             res |= 128;
-        } 
-    }    
-    
+        }
+    }
+
+    if (isRunning("mythtv-setup"))
+    {
+        VERBOSE(VB_IMPORTANT, "Setup is running...");
+        res = 255;
+    }
+
     VERBOSE(VB_GENERAL, "Mythshutdown: --status returned: " << res);
 
     return res;
 }
 
-int checkOKShutdown()
+int checkOKShutdown(bool bWantRecStatus)
 {
     // mythbackend wants 0=ok to shutdown, 1=reset idle count, 2=wait for frontend 
 
     VERBOSE(VB_GENERAL, "Mythshutdown: --check");
-    
-    int res = getStatus();        
-    
+
+    int res = getStatus(bWantRecStatus);
+
     if (res > 0)
-    {    
+    {
         VERBOSE(VB_IMPORTANT, "Not OK to shutdown");
         res = 1;
     }
     else
-    {    
+    {
         VERBOSE(VB_IMPORTANT, "OK to shutdown");
         res = 0;
-    }         
-    
+    }
+
     VERBOSE(VB_GENERAL, "Mythshutdown: --check returned: " << res);
 
     return res;
@@ -212,15 +339,15 @@ int checkOKShutdown()
 int setWakeupTime(QString sWakeupTime)
 {
     VERBOSE(VB_GENERAL, "Mythshutdown: --setwakeup");
-    
+
     VERBOSE(VB_IMPORTANT, "Mythshutdown: wakeup time given is: " << sWakeupTime);
-    
+
     // check time given is valid
     QDateTime dtWakeupTime;
     dtWakeupTime = QDateTime::fromString(sWakeupTime, Qt::ISODate);
-    
+
     if (!dtWakeupTime.isValid())
-    {    
+    {
         VERBOSE(VB_IMPORTANT, QString("Mythshutdown: --setwakeup invalid date "
                                       "format (%1)\n\t\t\t"
                                       "must be yyyy-MM-ddThh:mm:ss")
@@ -233,10 +360,89 @@ int setWakeupTime(QString sWakeupTime)
     return 0;
 }
 
+int setScheduledWakeupTime()
+{
+    typedef struct
+    {
+        QString channel, title, subtitle;
+        QDateTime startTime, endTime;
+    } ProgramDetail;
+
+    if (!gContext->IsConnectedToMaster())
+    {
+        VERBOSE(VB_IMPORTANT, "setScheduledWakeupTime: Attempting to connect to master server...");
+        if (!gContext->ConnectToMasterServer(false))
+        {
+            VERBOSE(VB_IMPORTANT, "setScheduledWakeupTime: Could not connect to master server!");
+            return false;
+        }
+    }
+
+    QDateTime m_nextRecordingStart = QDateTime();
+
+    ProgramList *progList = new ProgramList(true);
+    ProgramInfo *progInfo;
+
+    if (progList->FromScheduler())
+    {
+        if (progList->count() > 0)
+        {
+            VERBOSE(VB_IMPORTANT, "setScheduledWakeupTime: At least one scheduled recording found.");
+            // find the earliest scheduled recording
+            for (progInfo = progList->first(); progInfo; progInfo = progList->next())
+            {
+                if (progInfo->recstatus == rsWillRecord)
+                {
+                    if (m_nextRecordingStart.isNull() ||
+                            m_nextRecordingStart > progInfo->recstartts)
+                    {
+                        m_nextRecordingStart = progInfo->recstartts;
+                    }
+                }
+            }
+
+            // save the details of the earliest recording(s)
+            for (progInfo = progList->first(); progInfo; progInfo = progList->next())
+            {
+                if (progInfo->recstatus == rsWillRecord)
+                {
+                    if (progInfo->recstartts == m_nextRecordingStart)
+                    {
+                        ProgramDetail *prog = new ProgramDetail;
+                        prog->channel = progInfo->channame;
+                        prog->title = progInfo->title;
+                        prog->subtitle = progInfo->subtitle;
+                        prog->startTime = progInfo->recstartts;
+                        prog->endTime = progInfo->recendts;
+                    }
+                }
+            }
+        }
+    }
+
+    delete progList;
+
+    // set the wakeup time for the next scheduled recording
+    if (!m_nextRecordingStart.isNull())
+    {
+        int m_preRollSeconds = gContext->GetNumSetting("RecordPreRoll");
+        QDateTime restarttime = m_nextRecordingStart.addSecs((-1) * m_preRollSeconds);
+
+        int add = gContext->GetNumSetting("StartupSecsBeforeRecording", 240);
+        if (add)
+            restarttime = restarttime.addSecs((-1) * add);
+
+        setWakeupTime(restarttime.toString(Qt::ISODate));
+
+        return true;
+    }
+    return false;
+}
+
 int shutdown()
 {
     VERBOSE(VB_GENERAL, "Mythshutdown: --shutdown");
-    
+
     // get next daily wakeup times if any are set
     QDateTime dtPeriod1Start = getDailyWakeupTime("DailyWakeupStartPeriod1");
     QDateTime dtPeriod1End = getDailyWakeupTime("DailyWakeupEndPeriod1");
@@ -244,7 +450,24 @@ int shutdown()
     QDateTime dtPeriod2End = getDailyWakeupTime("DailyWakeupEndPeriod2");
     QDateTime dtCurrent = QDateTime::currentDateTime();
     QDateTime dtNextDailyWakeup = QDateTime();
-    
+
+    // Check for time periods that cross midnight
+    if (dtPeriod1End < dtPeriod1Start)
+    {
+        if (dtCurrent > dtPeriod1End)
+            dtPeriod1End = dtPeriod1End.addDays(1);
+        else
+            dtPeriod1Start = dtPeriod1Start.addDays(-1);
+    }
+
+    if (dtPeriod2End < dtPeriod2Start)
+    {
+        if (dtCurrent > dtPeriod2End)
+            dtPeriod2End = dtPeriod2End.addDays(1);
+        else
+            dtPeriod2Start = dtPeriod2Start.addDays(-1);
+    }
+
     // have we passed the first wakeup time today
     if (dtPeriod1Start != dtPeriod1End)
     {
@@ -253,7 +476,7 @@ int shutdown()
             VERBOSE(VB_IMPORTANT, "daily wakeup today at " << 
                         dtPeriod1Start.toString("hh:mm:ss"));
             dtNextDailyWakeup = dtPeriod1Start;
-        } 
+        }
     }
 
     // have we passed the second wakeup time today
@@ -264,9 +487,9 @@ int shutdown()
             VERBOSE(VB_IMPORTANT, "daily wakeup today at " << 
                         dtPeriod2Start.toString("hh:mm:ss"));
             dtNextDailyWakeup = dtPeriod2Start;
-        } 
+        }
     }
-    
+
     // if we have at least one valid daily wakeup time and dtNextDailyWakeup is 
     // still not valid then next daily wakeup is tomorrow 
     if (!dtNextDailyWakeup.isValid())
@@ -275,16 +498,16 @@ int shutdown()
             dtNextDailyWakeup = dtPeriod1Start;
         else if (dtPeriod2Start != dtPeriod2End)
             dtNextDailyWakeup = dtPeriod2Start;
-        
+
         if (dtNextDailyWakeup.isValid())
         {
-            dtNextDailyWakeup = dtNextDailyWakeup.addDays(1);    
-            
+            dtNextDailyWakeup = dtNextDailyWakeup.addDays(1);
+
             VERBOSE(VB_IMPORTANT, "next daily wakeup is tomorrow at " << 
                         dtNextDailyWakeup.toString("hh:mm:ss"));
         }
-    }      
-    
+    }
+
     // if dtNextDailyWakeup is still not valid then no daily wakeups are set
     if (!dtNextDailyWakeup.isValid())
         VERBOSE(VB_IMPORTANT,"no daily wakeup times are set");
@@ -292,15 +515,15 @@ int shutdown()
     // get next scheduled wake up for a recording if any
     QDateTime dtNextRecordingStart = QDateTime();
     QString s = getGlobalSetting("MythShutdownNextScheduled", "");
-    if (s != "")    
+    if (s != "")
         dtNextRecordingStart = QDateTime::fromString(s, Qt::ISODate);
-    
+
     if (!dtNextRecordingStart.isValid())
         VERBOSE(VB_IMPORTANT,"no recording time is set");
     else
         VERBOSE(VB_IMPORTANT, "recording scheduled at: " << 
-                    dtNextRecordingStart.toString(Qt::ISODate)); 
-      
+                    dtNextRecordingStart.toString(Qt::ISODate));
+
     // check if scheduled recording time has already passed
     if (dtNextRecordingStart.isValid())
     {
@@ -308,38 +531,38 @@ int shutdown()
 
         if (delta < 0) 
         {
-            VERBOSE(VB_IMPORTANT, "scheduled recording time has already passed. " 
+            VERBOSE(VB_IMPORTANT, "scheduled recording time has already passed. "
                                   "schedule deleted"); 
-        
+
             dtNextRecordingStart = QDateTime();
             setGlobalSetting("MythShutdownNextScheduled", "");
-        }    
+        }
     }
-     
+
     QDateTime dtWakeupTime = QDateTime();
-     
+
     // simple case 
     // no daily wakeup set
     // no scheduled program set
     // just shut down
     if (!dtNextRecordingStart.isValid() && !dtNextDailyWakeup.isValid())
-    {    
+    {
         dtWakeupTime = QDateTime();
         VERBOSE(VB_IMPORTANT, "no wake up time set and no scheduled program");
     }
-  
+
     // no daily wakeup set
     // scheduled program is set
     if (dtNextRecordingStart.isValid() && !dtNextDailyWakeup.isValid())
-    {    
+    {
         dtWakeupTime = dtNextRecordingStart;
         VERBOSE(VB_IMPORTANT, "will wake up at next scheduled program");
     }
-  
+
     // daily wakeup is set
     // no scheduled program is set
     if (!dtNextRecordingStart.isValid() && dtNextDailyWakeup.isValid())
-    {    
+    {
         dtWakeupTime = dtNextDailyWakeup;
         VERBOSE(VB_IMPORTANT, "will wake up at next daily wakeup");
     }
@@ -348,115 +571,134 @@ int shutdown()
     // scheduled program is set
     // wake up at which ever is the earliest
     if (dtNextRecordingStart.isValid() && dtNextDailyWakeup.isValid())
-    {    
+    {
         if (dtNextDailyWakeup < dtNextRecordingStart)
         {
             VERBOSE(VB_IMPORTANT,  "program is scheduled but will wake up "
                                    "at next daily wakeup");
             dtWakeupTime = dtNextDailyWakeup;
-        }    
+        }
         else
-        {    
+        {
             VERBOSE(VB_IMPORTANT, "daily wakeup is set but will wake up "
                                   "at next scheduled program");
             dtWakeupTime = dtNextRecordingStart;
         }
     }
-    
+
     // save the next wakuptime in the db
     setGlobalSetting("MythShutdownWakeupTime", dtWakeupTime.toString(Qt::ISODate));
-    
+
     // stop here to debug  
     //return 0;
-      
+
     int shutdownmode = 0; // default to poweroff no reboot 
+    QString nvramRestartCmd = 
+            gContext->GetSetting("MythShutdownNvramRestartCmd", "");
 
     if (dtWakeupTime.isValid())
     {
         // dont't shutdown if we are within 15 mins of the next wakeup time
         if (dtCurrent.secsTo(dtWakeupTime) > 15 * 60)
-        {    
+        {
             QString nvramCommand = gContext->GetSetting("MythShutdownNvramCmd",
-                     "/usr/bin/nvram-wakeup");
-            nvramCommand += QString(" --settime %1")
-                            .arg(dtWakeupTime.toTime_t());
-                              
+                     "/usr/bin/nvram-wakeup --settime $time");
+
+            QString wakeup_timeformat = gContext->GetSetting("MythShutdownWakeupTimeFmt", "time_t");
+
+            if (wakeup_timeformat == "time_t")
+            {
+                QString time_ts;
+                nvramCommand.replace("$time", time_ts.setNum(dtWakeupTime.toTime_t()));
+            }
+            else
+                nvramCommand.replace("$time", dtWakeupTime.toString(wakeup_timeformat));
+
             VERBOSE(VB_IMPORTANT, QString("sending command to set time in bios\n\t\t\t%1")
                             .arg(nvramCommand));
-            
-            shutdownmode = system(nvramCommand);          
+
+            shutdownmode = system(nvramCommand);
             if (WIFEXITED(shutdownmode))
                 shutdownmode = WEXITSTATUS(shutdownmode);
 
             VERBOSE(VB_IMPORTANT, QString("%1 exited with code %2").arg(nvramCommand)
-                                                             .arg(shutdownmode)); 
-            VERBOSE(VB_IMPORTANT, "Currently we don't trust exit code 0, forcing to 1..."); 
-            shutdownmode = 1;
-        }    
+                                                             .arg(shutdownmode));
+
+            if (shutdownmode == 2)
+            {
+                VERBOSE(VB_IMPORTANT, "nvram-wakeup failed to set time in bios");
+                return 1;
+            }
+
+            // we don't trust the return code from nvram-wakeup so only reboot
+            // if the user has set a restart command
+            if (nvramRestartCmd == "")
+                shutdownmode = 0;
+            else
+                shutdownmode = 1;
+        }
         else
-        {    
-            VERBOSE(VB_IMPORTANT, "The next wakeup time is less than 15 mins away," 
-                                  "not shutting down"); 
-            return 0;  
-        }   
-    } 
+        {
+            VERBOSE(VB_IMPORTANT, "The next wakeup time is less than 15 mins away,"
+                                  "not shutting down");
+            return 0;
+        }
+    }
 
     int res = 0;
-    
+
     switch (shutdownmode) 
     {
         case 0:
-        { 
-            VERBOSE(VB_IMPORTANT, "everything looks fine, shutting down ..."); 
-            QString poweroffCmd = 
+        {
+            VERBOSE(VB_IMPORTANT, "everything looks fine, shutting down ...");
+            QString poweroffCmd =
                     gContext->GetSetting("MythShutdownPoweroff", "/sbin/poweroff");
-            VERBOSE(VB_IMPORTANT, ".."); 
-            VERBOSE(VB_IMPORTANT, "."); 
-            VERBOSE(VB_IMPORTANT, "shutting down ..."); 
+            VERBOSE(VB_IMPORTANT, "..");
+            VERBOSE(VB_IMPORTANT, ".");
+            VERBOSE(VB_IMPORTANT, "shutting down ...");
 
-            system(poweroffCmd); 
+            system(poweroffCmd);
             res = 0;
             break;
         }
-        case 1:     
-        {    
-            QString nvramRestartCmd = 
-                    gContext->GetSetting("MythShutdownNvramRestartCmd", "grub-set-default 1");
-            VERBOSE(VB_IMPORTANT, "everything looks fine, but reboot is needed"); 
-            VERBOSE(VB_IMPORTANT, "sending command to bootloader ..."); 
+        case 1:
+        {
+            VERBOSE(VB_IMPORTANT, "everything looks fine, but reboot is needed");
+            VERBOSE(VB_IMPORTANT, "sending command to bootloader ...");
             VERBOSE(VB_IMPORTANT, nvramRestartCmd);
-            
+
             system(nvramRestartCmd);
-            
-            VERBOSE(VB_IMPORTANT, ".."); 
-            VERBOSE(VB_IMPORTANT, "."); 
-            VERBOSE(VB_IMPORTANT, "rebooting ..."); 
-            
-            QString rebootCmd = 
+
+            VERBOSE(VB_IMPORTANT, "..");
+            VERBOSE(VB_IMPORTANT, ".");
+            VERBOSE(VB_IMPORTANT, "rebooting ...");
+
+            QString rebootCmd =
                     gContext->GetSetting("MythShutdownReboot", "/sbin/reboot");
             system(rebootCmd);
             res = 0;
             break;
         }
-        default:     
-            VERBOSE(VB_IMPORTANT, "panic. invalid shutdown mode, do nothing"); 
-            res = 1; 
+        default:
+            VERBOSE(VB_IMPORTANT, "panic. invalid shutdown mode, do nothing");
+            res = 1;
             break;
-    }        
-    
+    }
+
     return res;
 }
 
 int startup()
 {
     VERBOSE(VB_GENERAL, "Mythshutdown: --startup");
-    
+
     int res = 0;
     QDateTime startupTime = QDateTime();
     QString s = getGlobalSetting("MythshutdownWakeupTime", "");
     if (s != "")
         startupTime = QDateTime::fromString(s, Qt::ISODate);
-    
+
     // if we don't have a valid startup time assume we were started manually
     if (!startupTime.isValid())
         res = 1;
@@ -464,69 +706,70 @@ int startup()
     {
         // if we started within 15mins of the saved wakeup time assume we started
         // automatically to record or for a daily wakeup/shutdown period 
-        
+
         int delta = startupTime.secsTo(QDateTime::currentDateTime());
         if (delta < 0)
             delta = -delta;
-            
+
         if (delta < 15 * 60)
             res = 0;
         else
-            res = 1;        
+            res = 1;
     }
-        
+
     if (res)
         VERBOSE(VB_IMPORTANT, "looks like we were started manually" << res);
     else
         VERBOSE(VB_IMPORTANT, "looks like we were started automatically" << res);
 
-    
+
     VERBOSE(VB_GENERAL, "Mythshutdown: --startup returned: " << res);
-    
+
     return res;
 }
 
 void showUsage()
 {
   cout << "Usage of mythshutdown\n"; 
-  cout << "-w/--setwakeup time (sets the wakeup time. time=yyyy-MM-ddTmm:hh:ss.\n"; 
-  cout << "                     doesn't write it into nvram)\n";  
-  cout << "-q/--shutdown       (set nvram-wakeup time and shutdown)\n"; 
-  cout << "-p/--startup        (check startup. check will return 0 if automatic\n";
-  cout << "                                                      1 for manually)\n"; 
-  cout << "-c/--check          (check shutdown possible returns 0 ok to shutdown\n"; 
-  cout << "                                                     1 reset idle check)\n"; 
-  cout << "-l/--lock           (disable shutdown. check will return 1.)\n"; 
-  cout << "-u/--unlock         (enable shutdown. check will return 0)\n";
-  cout << "-s/--status         (returns a code indicating the current status)\n";
-  cout << "         0 - Idle\n";
-  cout << "         1 - Transcoding\n";
-  cout << "         2 - Commercial Flagging\n";
-  cout << "         4 - Grabbing EPG data\n";
-  cout << "         8 - Not used\n";
-  cout << "        16 - Locked\n";
-  cout << "        32 - Not used\n";
-  cout << "        64 - In a daily wakeup/shutdown period\n";
-  cout << "       128 - Less than 15 minutes to next wakeup period\n";
-  cout << "-v/--verbose debug-level  (Use '-v help' for level info\n";
-  cout << "-h/--help                 (shows this usage)\n";
+  cout << "-w/--setwakeup time      (sets the wakeup time. time=yyyy-MM-ddThh:mm:ss\n";
+  cout << "                          doesn't write it into nvram)\n";
+  cout << "-t/--setscheduledwakeup  (sets the wakeup time to the next scheduled recording)\n";
+  cout << "-q/--shutdown            (set nvram-wakeup time and shutdown)\n";
+  cout << "-x/--safeshutdown        (equal to -c -t -q.  check shutdown possible, set\n";
+  cout <<"                           scheduled wakeup and shutdown)\n";
+  cout << "-p/--startup             (check startup. check will return 0 if automatic\n";
+  cout << "                                                           1 for manually)\n";
+  cout << "-c/--check flag          (check shutdown possible\n"; 
+  cout << "                          flag is 0 - don't check recording status\n";
+  cout << "                                  1 - do check recording status (default)\n";
+  cout << "                          returns 0 ok to shutdown\n";
+  cout << "                                  1 reset idle check)\n";
+  cout << "-l/--lock                (disable shutdown. check will return 1.)\n";
+  cout << "-u/--unlock              (enable shutdown. check will return 0)\n";
+  cout << "-s/--status flag         (returns a code indicating the current status)\n";
+  cout << "                          flag is 0 - don't check recording status\n";
+  cout << "                                  1 - do check recording status (default)\n";
+  cout << "                          0 - Idle\n";
+  cout << "                          1 - Transcoding\n";
+  cout << "                          2 - Commercial Flagging\n";
+  cout << "                          4 - Grabbing EPG data\n";
+  cout << "                          8 - Recording - only valid if flag is 1\n";
+  cout << "                         16 - Locked\n";
+  cout << "                         32 - Jobs running or pending\n";
+  cout << "                         64 - In a daily wakeup/shutdown period\n";
+  cout << "                        128 - Less than 15 minutes to next wakeup period\n";
+  cout << "                        255 - Setup is running\n";
+  cout << "-v/--verbose debug-level (Use '-v help' for level info\n";
+  cout << "-h/--help                (shows this usage)\n";
 }
 
 int main(int argc, char **argv)
 {
     // by default we don't output any messages
     print_verbose_messages = VB_NONE;
-    
+
     QApplication a(argc, argv, false);
-        
-    gContext = NULL;
-    gContext = new MythContext(MYTH_BINARY_VERSION);
-    if (!gContext->Init(false))
-    {
-        cout << "mythshutdown: Could not initialize myth context. "
-                "Exiting." << endl;;
-        return FRONTEND_EXIT_NO_MYTHCONTEXT;
-    }
+
 
     bool bLockShutdown = false;
     bool bUnlockShutdown = false;
@@ -537,7 +780,10 @@ int main(int argc, char **argv)
     bool bSetWakeupTime = false;
     QString sWakeupTime = "";
     bool bShowUsage = false;
-        
+    bool bSetScheduledWakeupTime = false;
+    bool bCheckAndShutdown = false;
+    bool bWantRecStatus = true;
+
     //  Check command line arguments
     for (int argpos = 1; argpos < a.argc(); ++argpos)
     {
@@ -548,9 +794,9 @@ int main(int argc, char **argv)
             {
                 if (parse_verbose_arg(a.argv()[argpos+1]) ==
                         GENERIC_EXIT_INVALID_CMDLINE)
-                    return FRONTEND_EXIT_INVALID_CMDLINE;                        
+                    return FRONTEND_EXIT_INVALID_CMDLINE;
                 ++argpos;
-            } 
+            }
             else
             {
                 cerr << "Missing argument to -v/--verbose option\n";
@@ -572,11 +818,31 @@ int main(int argc, char **argv)
             !strcmp(a.argv()[argpos],"--check"))
         {
             bCheckOKShutdown = true;
+            if (a.argc() - 1 > argpos)
+            {
+                QString s = a.argv()[argpos+1];
+                if (!s.startsWith("-"))
+                {
+                    if (s == "0")
+                        bWantRecStatus = false;
+                    ++argpos;
+                }
+            }
         }
         else if (!strcmp(a.argv()[argpos],"-s") ||
             !strcmp(a.argv()[argpos],"--status"))
         {
             bGetStatus = true;
+            if (a.argc() - 1 > argpos)
+            {
+                QString s = a.argv()[argpos+1];
+                if (!s.startsWith("-"))
+                {
+                    if (s == "0")
+                        bWantRecStatus = false;
+                    ++argpos;
+                }
+            }
         }
         else if (!strcmp(a.argv()[argpos],"-w") ||
             !strcmp(a.argv()[argpos],"--setwakeup"))
@@ -610,6 +876,17 @@ int main(int argc, char **argv)
         {
             bShowUsage = true;
         }
+        else if (!strcmp(a.argv()[argpos],"-t") ||
+            !strcmp(a.argv()[argpos],"--setscheduledwakeup"))
+        {
+            bSetScheduledWakeupTime = true;
+        }
+        else if (!strcmp(a.argv()[argpos],"-x") ||
+            !strcmp(a.argv()[argpos],"--safeshutdown"))
+        {
+            bCheckAndShutdown = true;
+        }
+
         else
         {
             cout << "Invalid argument: " << a.argv()[argpos] << endl;
@@ -617,9 +894,19 @@ int main(int argc, char **argv)
             return FRONTEND_EXIT_INVALID_CMDLINE;
         }
     }
-    
+
+
+    gContext = new MythContext(MYTH_BINARY_VERSION);
+    if (!gContext->Init(false))
+    {
+        cout << "mythshutdown: Could not initialize myth context. "
+                "Exiting." << endl;
+        return FRONTEND_EXIT_NO_MYTHCONTEXT;
+    }
+
+
     int res = 0;
-    
+
     if (bShowUsage)
         showUsage();
     else if (bLockShutdown)
@@ -627,18 +914,31 @@ int main(int argc, char **argv)
     else if (bUnlockShutdown)
         res = unlockShutdown();
     else if (bCheckOKShutdown)
-        res = checkOKShutdown();
+        res = checkOKShutdown(bWantRecStatus);
+    else if (bSetScheduledWakeupTime)
+        res = setScheduledWakeupTime();
     else if (bStartup)
         res = startup();
     else if (bShutdown)
         res = shutdown();
     else if (bGetStatus)
-        res = getStatus();
+        res = getStatus(bWantRecStatus);
     else if (bSetWakeupTime)
         res = setWakeupTime(sWakeupTime);
-    else 
+    else if (bCheckAndShutdown)
+    {
+        res = checkOKShutdown(true);
+        if (res == 0)     // Nothing to stop a shutdown (eg. recording in progress).
+        {
+             res = setScheduledWakeupTime();
+             res = shutdown();
+        }
+    }
+    else
         showUsage();
-    
+
+    delete gContext;
+
     return res;
 }
 
