@@ -1,9 +1,14 @@
+// ANSI C headers
 #include <cstdlib>
+
+// POSIX headers
 #include <sys/types.h>
-#include <sys/socket.h>
 
+// C++ headers
 #include <iostream>
+using namespace std;
 
+// Qt headers
 #include <qapplication.h>
 #include <qsocketdevice.h>
 #include <qstring.h>
@@ -11,9 +16,9 @@
 #include <qfile.h>
 #include <qhostaddress.h>
 
+// MythTV headers
 #include "exitcodes.h"
-
-using namespace std;
+#include "compat.h"
 
 const QString kalert =
 "<mythnotify version=\"1\">\n"
@@ -171,11 +176,12 @@ int main(int argc, char *argv[])
         QString name = arg.section("=", 0, 0);
         name.replace("--", "");
 
-        QString value = arg.section("=", 1);
+        QString value = QString::fromLocal8Bit(arg.section("=", 1));
         if (verbose)
         {
 
-            cerr << "name: " << name << " -- value: " << value << endl;
+            cerr << "name: " << name 
+		 << " -- value: " << value.local8Bit() << endl;
         }
 
         name.append("%");
@@ -185,7 +191,7 @@ int main(int argc, char *argv[])
     }
 
     if (verbose)
-        cout << "output:\n" << message << endl;
+        cout << "output:\n" << message.local8Bit() << endl;
 
     QSocketDevice sock(QSocketDevice::Datagram);
 
